

/*  Copyright (C) 2002 Secure Systems Lab, Stony Brook. 

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program (See file COPYING in the main directory); 
    if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*   Written by Tapan Kamat tkamat@cs.sunysb.edu */


import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;

/**
 * A subclass of JTree that allows individual nodes to have checkboxes.
 * All that is required to have a node display a checkbox is for the user object
 * that that is used to create the DefaultMutableTreeNode to implement the
 * Checkable interface. When the user selects or deselects the node in the
 * tree the objects setChecked(boolean v) method will be called.
 *
 * @author 
 */
public class CheckTree extends JTree {

  /**
   * Create a CheckTree object from the given TreeModel.
   *
   * @param tm The TreeModel that should be used to create the tree.
   */
  public CheckTree(TreeModel tm) {
    super(tm);
    this.setCellRenderer(CheckableTreeCellRenderer.getStaticInstance());
    addCheckListener();
  }

  /**
   * Create a CheckTree object using the default Swing tree model.
   */
  public CheckTree() {
    super();
    this.setCellRenderer(CheckableTreeCellRenderer.getStaticInstance());
    addCheckListener();
  }

  //MFW! - START CHANGED LINES
  private ArrayList listenerList;

  public void addCheckListener(CheckListener l) {
    if(listenerList == null) {
      listenerList = new ArrayList();
    }
    listenerList.add(l);
  }

  public void removeCheckListener(CheckListener l) {
    if(listenerList == null) {
      return;
    }
    listenerList.remove(l);
  }

  protected void fireCheckEvent(CheckEvent evt) {
    if(listenerList == null) {
      return;
    }
    CheckListener[] listeners = (CheckListener[])listenerList.toArray(new CheckListener[listenerList.size()]);
    for(int i = 0; i < listeners.length;i++) {
      listeners[i].valueChanged(evt);
    }
  }
  //MFW! - END CHANGED LINES


  /**
   * Adds a private listener used to update the status of the checkboxes
   */
  private void addCheckListener() {
    MouseListener ml = new MouseAdapter() {
      public void mousePressed(MouseEvent e) {
          TreePath selPath = getPathForLocation(e.getX(), e.getY());
          if(selPath != null && isClickInBox(e)) {
              if(e.getClickCount() == 1 || e.getClickCount() == 2) {
                  TreeNode tNode = (TreeNode) selPath.getLastPathComponent();
                  if(tNode instanceof DefaultMutableTreeNode) {
                    DefaultMutableTreeNode dMTN = (DefaultMutableTreeNode)tNode;
                    Object o = dMTN.getUserObject();
                    if(o instanceof Checkable) {
                      Checkable checkable = (Checkable)o;
                      checkable.setChecked(!(checkable.isChecked()));
                      //MFW! - START CHANGED LINES
                      CheckEvent evt = new CheckEvent(dMTN, checkable.isChecked());
                      fireCheckEvent(evt);
                      //MFW! - END CHANGED LINES
                    }
                    repaint();
                  }
              }
          }
      }
    };
    addMouseListener(ml);
  }

  /**
   * Checks to see if the given mouse event occured within the checkbox of the
   * component that was selected.
   */
  private boolean isClickInBox(MouseEvent event) {
    if(event != null) {
      Point p = event.getPoint();
      int selRow = getRowForLocation(p.x, p.y);
      TreeCellRenderer r = getCellRenderer();

      if(selRow != -1 && r != null) {
        TreePath path = getPathForRow(selRow);
        Object       lastPath = path.getLastPathComponent();
        Component    rComponent = r.getTreeCellRendererComponent(this, lastPath, isRowSelected(selRow),
                       isExpanded(selRow), getModel().isLeaf(lastPath), selRow,
                       true);

        if(rComponent instanceof CheckableTreeCellRenderer) {
          CheckableTreeCellRenderer cTCR = (CheckableTreeCellRenderer)rComponent;

          Rectangle pathBounds = getPathBounds(path);
          p.translate(-pathBounds.x, -pathBounds.y);
          return cTCR.isPointInBox(p);
        }
      }
    }
    return false;
  }
}
