

/*  Copyright (C) 2002 Secure Systems Lab, Stony Brook. 

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program (See file COPYING in the main directory); 
    if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*   Written by Tapan Kamat tkamat@cs.sunysb.edu */

import java.io.*;
import java.util.*;
import javax.swing.tree.*;

import java.awt.*;
import javax.swing.*;

/** Aim : A class which implements some commonly used string parsing and other methods suited to achieve a specific functionality
    Author : Tapan Kamat 
 */


public class CommonMethods
{      
    // Method to determine if a particular violation is to be displayed or not. 
    // If a violation is not be displayed it is supposed to not exist at all. 
    public static boolean displayViolation(String violation)
    {	
	violation = violation.toUpperCase();
	System.out.println("Violation caught is : "  + violation  + "    Length is : " + violation.length());
	if ((new String("CREATE IN NOT OWNED DIRECTORIES").startsWith(violation)) && (GUI.jcbCreateNotOwn.isSelected())) return false;
	if ((new String("WRITE TO OWNED BUT MODIFIED FILES").startsWith(violation)) && (GUI.jcbWriteOwnMod.isSelected())) return false;
	if ((new String("WRITE TO FILES IN NOT OWNED DIRECTORIES").startsWith(violation)) && (GUI.jcbWriteNotOwn.isSelected())) return false;
	if ((new String("UNAUTHORIZED FILE WRITE").startsWith(violation)) && (GUI.jcbWriteShared.isSelected())) return false;
	if ((new String("ADD USERS").startsWith(violation)) && (GUI.jcbAddUsers.isSelected())) return false;
	if ((new String("UPDATE SYSTEM SERVICES").startsWith(violation)) && (GUI.jcbUpdateSysServices.isSelected())) return false;
	if ((new String("MODIFY SHARED LIBRARIES").startsWith(violation)) && (GUI.jcbModifyLibs.isSelected())) return false;
	if ((new String("PERFORM NETWORK OPERATIONS").startsWith(violation)) && (GUI.jcbPerformNetworkOps.isSelected())) return false;
	if ((new String("DELETE FILES").startsWith(violation)) && (GUI.jcbDeleteFiles.isSelected())) return false;
	if ((new String("UNAUTHORIZED COMMAND EXECUTION").startsWith(violation)) && (GUI.jcbExecuteUnCmd.isSelected())) return false;
	return true;
    }
  
    // This method runs a script to create a symlink when given the path of the package being installed / analyzed
    public static void createSymLink(String path)
    {
	String prefix = CommonMethods.getPackageNameWithVersion(path);
	System.out.println("Prefix : " + prefix);
	try 
	{
	    Runtime theSystem = Runtime.getRuntime();
	    String execStr = "/bin/sh " + Constants.DEMODIR + "tools/makesymlinks " + prefix;
	    System.out.println(execStr);
	    Process proc = theSystem.exec(execStr);
	    proc.waitFor();
	    int val = proc.exitValue();
	    // Normal Termination (We get the result on the output stream)
	    if (val == 0) 
		System.out.println("Makesymlinks successful...");
	    else
		System.out.println("Makesymlinks failed...");
	}
	catch(InterruptedException e)
	{
	    System.out.println("Interrupted Exception in proc.waitFor in makeSysmLinks : "  + e.getMessage());
	}
	catch(Exception e)
	{
	    System.out.println("Exception in method in makeSymLinks : "  + e.getMessage()); 
	}	
    }

    // This method is called just before the installation commences under the tracer.
    // The purpose of this method is to check if the test install still would go thru and if not , report the 
    // packages that would be required to be selected for the 
    public static boolean testInstall(Vector accepts, Vector rejects)
    {	
	String msg = "";

	if (accepts.isEmpty())
	{
	    msg = "No files selected for installation.\nPlease select files for Installation  or Abort Installation.";
	    GUI.pauseProgressBar();
	    JOptionPane.showMessageDialog(null,msg, "No files selected",JOptionPane.INFORMATION_MESSAGE);		
	    return false;	  
	}  

	String selectedFiles = "";
	for (int i = 0; i < accepts.size(); i++)
	    selectedFiles = selectedFiles + " " + (String) accepts.elementAt(i); 

	String rejectedFiles = "";
	for (int i = 0; i < rejects.size(); i++)
	    rejectedFiles = rejectedFiles + " " + (String) rejects.elementAt(i); 

	int flag = 0;
	try
	{
	    Runtime theSystem = Runtime.getRuntime();
	    String execStr = "rpm -i --test " + selectedFiles;
	    Process proc = theSystem.exec(execStr);
	    proc.waitFor();
	    int val = proc.exitValue();		
		    
	    // After the process exec returns (finishes) along with an error message 
	    if (val > 0)
	    {
		BufferedReader buf = new BufferedReader(new InputStreamReader(proc.getErrorStream()));
		String s ;
		try
		{
		    while ((s = buf.readLine()) != null)	
		    {		
			// remove off the trailing or leading spaces , if any 
			s = s.trim();
			
			// first line contains the string "error" always in case of failed dependencies , so ignore it 
			if (s.startsWith("error:"))
			    continue;	
			
			// In case of failed installation, get the package name for which the test install failed
			int indexofneeded = s.indexOf(" is needed by");
			if (indexofneeded >= 0)
			{	
			    // set this flag to signify that we got dependencies problem
			    flag = 1;

			    // get only the name of the package
			    String pkgname = s.substring(0,s.indexOf(' '));
			    pkgname = pkgname.trim();

			    // this is a list of all the packages which the reject list will install
			    Vector list = CommonMethods.getPartialPackageName(rejectedFiles);
			    if (list.contains(pkgname))
			    {
				int indexofprovider = list.indexOf(pkgname);
				msg = msg + "\n" + pkgname + " : " + (String) rejects.elementAt(indexofprovider);
			    }
			}
		    }
		}
		catch(IOException e)
		{
		    System.out.println("IO Exception in readline in testInstall: " + e.getMessage());
		}
		buf.close();
	    }
	    // No error message given , so installation can go thru
	    else
	    {		   
		System.out.println("Val returned  <= 0"); 
		msg = "Test installation of selected packages was successful";
	    } 		
	}
	catch(InterruptedException e)
	{
	    System.out.println("Interrupted Exception in proc.waitFor in testInstall : "  + e.getMessage());
	}
	catch(Exception e)
	{
	    System.out.println("Exception in method testInstall : "  + e.getMessage()); 
	}
	finally
	{
	    // Some dependencies are found
	    if (flag == 1)
	    {
		String header = "One or more packages selected for installation require the following packages to be selected.\n"; 
		String footer = "\n\nPlease select the above packages for the installation to be succeed.\n";
		msg = header + msg + footer;
		GUI.pauseProgressBar();
		JOptionPane.showMessageDialog(null,msg, "Test Installation",JOptionPane.INFORMATION_MESSAGE);
		return false;
	    }		    
	    // No dependencies were found, so its safe to continue;
	    return true;
	}
    } 


    // Method to find if a file has been modified since installation or if the file which will be installed is 
    // different from the already present file
    public static boolean hasChanged(String filename) 
    {
	if ((filename == null) || (filename.equals("")))  
	    return false;

	boolean isChanged = false;      	
	File packageFile = new File(filename);
	if (packageFile.isDirectory() || (!packageFile.exists()))
	{
	    isChanged = false;
	    return false;
	} 
        //System.out.println("Microseconds Time from Unix : " + packageFile.lastModified());
	long lastModTime  = packageFile.lastModified()/1000; 

	try 
	{
	    Runtime theSystem = Runtime.getRuntime();
	    String execStr = "rpm -qf --queryformat %{INSTALLTIME} " + filename; 
	    //System.out.println(execStr);
	    Process proc = theSystem.exec(execStr);
	    proc.waitFor();
	    int val = proc.exitValue();

	    if (val == 0)
	    {
		BufferedReader buf = new BufferedReader(new InputStreamReader(proc.getInputStream()));
		String s;
		while ((s = buf.readLine()) != null) 
		{
		    long l = ((Integer.valueOf(s)).longValue());		    

		    //System.out.println("Time in seconds from Unix : " + lastModTime);
		    //System.out.println("Time from RPM Database  : " + l);
		    
		    // The file was modified after installation
		    if (l < lastModTime)
		    {
			System.out.println("The file : " + filename + "    changed between installations");
			isChanged = true;
		    }
		    // The file was not modified after installation
		    else 
		    {
			//System.out.println("The file did not change between installations.");
			isChanged = false;
		    }
		}
		buf.close();
	    }
	    else
	    {
		//System.out.println("File / directory not found in system  : " + execStr);
	    }
	} // end of outer try
	catch(InterruptedException e) 
	{
	    System.out.println("Interrupted Thread exception in hasChanged method " + e.getMessage());
	}
	catch(IOException e)
        {
	    System.out.println("IOException Exception in method hasChanged : "  + e.getMessage());
        }	
        catch(Exception e) 
        {
	    System.out.println("Exception in hasChanged method " + e.getMessage());
        }
        finally
        {
	    return isChanged;
        }
    } //end hasChanged  method


    // This method just takes an input list of the files/ directories in a parent dir and the sorts them such that the directories are listed first
    public static void sortForDirectories(File [] list)
    {
	Vector newListVector = new Vector(list.length);
	int totalmatch = 0;
	for(int i = 0 ; i < list.length;  i++)
	{
	    // if the file object is a reference to a directory , add at the end of last added directory position 
	    if(list[i].isDirectory())
	    {
		newListVector.insertElementAt(list[i],totalmatch);
		totalmatch++;
	    }
	    // if the file object is a reference to a file or any other object, then add this at the end 
	    else 
		newListVector.addElement(list[i]);
	}
	for(int i = 0 ; i < newListVector.size() ; i++)
	    list[i] = (File) newListVector.elementAt(i);
    }
    
    // This method sorts the URLs in a given vector with the URLs matching the given version at the beginning and the remaining URLs 
    // at the end of the vector
    public static Vector sortURLs(Vector oldlistofurls , String version)
    {
	Vector newlistofurls = new Vector();
	String url;
	String filename ;
	int totalmatch = 0;
	System.out.println("\nVersion is : " + version);
	for(int i = 0 ; i < oldlistofurls.size();  i++)
	{
	    url = (String) oldlistofurls.elementAt(i);
	    filename = getFileName(url);

	    // we find a version number match in the filename , so bring this url to the top part of the vector
	    if(filename.indexOf(version) >= 0)
	    {
		newlistofurls.insertElementAt(url,totalmatch);
		//System.out.println(url + " added at posn " + totalmatch);
		totalmatch++;
	    }
	    // version number does not match , hence put this url at the bottom of the vector
	    else
	    {
		newlistofurls.addElement(url);
		//System.out.println(url + " added at posn " + (newlistofurls.size() - 1));
	    }
	}
	return newlistofurls;
    }

    // This method returns just the end filename  (eg. abc-1.2.3.i386.rpm) from a source path (ftp.rpmfind.net/linux/downloads/abc-1.2.3.i386.rpm)
    public static String getFileName(String path)
    {
	if(path == null)
	    return null;
	
	path = path.trim();
	int index = path.lastIndexOf("/");
	return path.substring(index + 1);
    }

    // This method returns an absolute path string from a Treepath.toString()
    public static String getPath(String selectedpath)
    {
	if (selectedpath == null)
	{
	    return null;
	}
	
	if (selectedpath.indexOf(",") < 0)
	{
	    return "/";
	}
	String path = "";
	
	StringTokenizer strtok = new StringTokenizer(selectedpath,",");
	strtok.nextToken();
	while (strtok.hasMoreTokens())
	{
		path = path + strtok.nextToken().trim() + "/" ;
	}
	path = path.substring(0,path.length()-2);
	return path;	
    }

    // This method returns a String array of absolute filepaths given an array of TreePaths
    public static String[] getPaths(TreePath[] treepaths)
    {
	String selectedpaths[] = new String[treepaths.length]; 
	for (int i = 0; i < selectedpaths.length;i++)
	{	    
	    selectedpaths[i] = treepaths[i].toString();
	    String path = "/";

	    if (selectedpaths[i].indexOf(",") < 0)
	    {
		selectedpaths[i] = path;
		continue;
	    }
	    
	    StringTokenizer strtok = new StringTokenizer(selectedpaths[i],",");
	    strtok.nextToken();
	    while (strtok.hasMoreTokens())
	    {
		path = path + strtok.nextToken().trim() + "/" ;
	    }
	    selectedpaths[i] = path.substring(0,path.length()-2);	
	}
	return selectedpaths;
    }

    // This method queries the package header using the list of paths provided and returns only the package name for each particular package in an array
    public static Vector getPartialPackageName(String path) 
    {
	Vector partialNames = new Vector();
	try
	{
	    Runtime theSystem = Runtime.getRuntime();
	    String execStrArray[] = {"rpm", "-qp", "--queryformat","[%{NAME}\n]", path};
	    Process proc = theSystem.exec(execStrArray);
	    proc.waitFor();
	    int val = proc.exitValue();
	    BufferedReader buf = new BufferedReader(new InputStreamReader(proc.getInputStream()));
	    String s = "";
	    try
	    {
		while((s = buf.readLine())!= null)
		    partialNames.add(s);		    
	    }
	    catch(IOException e)
	    {
		System.out.println("IO Exception in readline : " + e.getMessage());
	    }
	    buf.close();
	}
	catch(InterruptedException e)
	{
	    System.out.println("Interrupted Exception in proc.waitFor in getPartialPackageName: "  + e.getMessage());
	}
	catch(Exception e)
	{
	    System.out.println("Exception in method getPartialPackageName : "  + e.getMessage()); 
	}	
	finally
	{
	    return partialNames;
	}
    }

    // This method queries the package header using the list of paths provided and returns the entire package name with version for each package in an array
    public static Vector getFullPackageName(String path) 
    {
	Vector fullNames = new Vector();
	try
	{
	    Runtime theSystem = Runtime.getRuntime();
	    String execStr = "rpm -qp " + path;
	    Process proc = theSystem.exec(execStr);
	    proc.waitFor();
	    int val = proc.exitValue();
	    BufferedReader buf = new BufferedReader(new InputStreamReader(proc.getInputStream()));
	    String s = ""; 
	    try
	    {
		while((s = buf.readLine())!= null)
		    fullNames.add(s);
	    }
	    catch(IOException e)
	    {
		System.out.println("IO Exception in readline : " + e.getMessage());
	    }
	    buf.close();
	}
	catch(InterruptedException e)
	{
	    System.out.println("Interrupted Exception in proc.waitFor in getFullPackageName: "  + e.getMessage());
	}
	catch(Exception e)
	{
	    System.out.println("Exception in method getFullPackageName : "  + e.getMessage()); 
	}	
	finally
	{
	    return fullNames;
	}
    }

    // This method queries the rpm database using the list of packagenames provided and returns the output in a Vector
    public static Vector queryRPMDatabase(String path) 
    {
	Vector result = new Vector();
	try
	{
	    Runtime theSystem = Runtime.getRuntime();
	    String execStr = "rpm -q " + path;
	    Process proc = theSystem.exec(execStr);
	    proc.waitFor();
	    int val = proc.exitValue();
	    BufferedReader buf = new BufferedReader(new InputStreamReader(proc.getInputStream()));
	    String s = ""; 
	    try
	    {
		while((s = buf.readLine())!= null)
		    result.add(s);
	    }
	    catch(IOException e)
	    {
		System.out.println("IO Exception in readline : " + e.getMessage());
	    }
	    buf.close();
	}
	catch(InterruptedException e)
	{
	    System.out.println("Interrupted Exception in proc.waitFor in  queryRPMDatabase: "  + e.getMessage());
	}
	catch(Exception e)
	{
	    System.out.println("Exception in method queryRPMDatabase : "  + e.getMessage()); 
	}	
	finally
	{
	    return result;
	}
    }
    
    // This method queries the RPM database using the path provided and returns the name of the file owner , if any 
    // Note that just the name of the file owner is returned and not the full version number
    public static Vector getOwner(String path) 
    {
	Vector owners = new Vector();
	try
	{
	    Runtime theSystem = Runtime.getRuntime();
	    String execStrArray[] = {"rpm", "-qf", "--queryformat","[%{NAME}\n]", path};
	    Process proc = theSystem.exec(execStrArray);
	    proc.waitFor();
	    int val = proc.exitValue();
	    BufferedReader buf = new BufferedReader(new InputStreamReader(proc.getInputStream()));
	    String s = ""; 
	    try
	    {
		while((s = buf.readLine())!= null)		  		       
		{
		    if (s.indexOf("not owned by any package") >= 0)
			break;
		    owners.add(s);
		}
	    }
	    catch(IOException e)
	    {
		System.out.println("IO Exception in readline : " + e.getMessage());
	    }
	    buf.close();
	}
	catch(InterruptedException e)
	{
	    System.out.println("Interrupted Exception in proc.waitFor in getOwner: "  + e.getMessage());
	}
	catch(Exception e)
	{
	    System.out.println("Exception in method getOwner : "  + e.getMessage()); 
	}	
	finally
	{
	    return owners;
	}
    }


    // This method returns a pkg name without the .rpm extension  when its passed a String containing the absolute/relative package path 
    public static String getPackageNameWithVersion(String filename)
    {
	String name = "";
	// if the path does not name with .rpm  then reject the package name returning empty String
	if(!filename.endsWith(".rpm"))
	    return name;

	// Absolute path
	if (filename.indexOf("/") >= 0)
	{
	    File file = new File (filename);
	    name = file.getName();
	    int lastindexslash = name.lastIndexOf("/");
	    name = name.substring(lastindexslash + 1);
	}
	else 
	    name = filename;
	int lastindexperiod = name.lastIndexOf(".rpm");
	if (lastindexperiod >= 0)
	    name = name.substring(0,lastindexperiod);
	else
	    name = "";
	return name;	    
    }

    // u are allowed to create the file so add the filename to the /tmp/create  file 
    public static void addToCreateFile(String packagename, String s)
    {
	try
	{
	    BufferedWriter createBuf = new BufferedWriter(new FileWriter("/tmp/" + packagename + "-create",true));
	    createBuf.write(s);
	    createBuf.close();		
	}
	catch(IOException e)
	{
	    System.out.println("IOException Exception in method addToCreateFile : "  + e.getMessage());
	}
	catch(Exception e)
	{
	    System.out.println("Exception in method addToCreateFile : "  + e.getMessage()); 
	}
    }

    // u are allowed to write to the file so add the filename to the /tmp/write  file 
    public static void addToWriteFile(String packagename, String s)
    {
	try
	{
	    BufferedWriter writeBuf = new BufferedWriter(new FileWriter("/tmp/" + packagename + "-write",true));
	    writeBuf.write(s);
	    writeBuf.close();		
	}
	catch(IOException e)
	{
	    System.out.println("IOException Exception in method addToWriteFile : "  + e.getMessage());
	}
	catch(Exception e)
	{
	    System.out.println("Exception in method addToWriteFile : "  + e.getMessage()); 
	}
    }

    public static String getExtension(File filename)
    {
	String name = filename.getName();
	int idx = name.lastIndexOf(".");
	if (idx <= 0)
	    return "";
	String extension = name.substring(idx+1);
	return extension;
    }	    

	
    public static void main(String args[])
    {
    }
}
