

/*  Copyright (C) 2002 Secure Systems Lab, Stony Brook. 

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program (See file COPYING in the main directory); 
    if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*   Written by Tapan Kamat tkamat@cs.sunysb.edu */

import java.io.*;
import java.util.*;
import java.net.*;
import javax.swing.JOptionPane;

/**
 * Program written by Tapan Kamat 
 * Functionality : To
 *                 installs successfully 
 *
 */

public class FetchDependencies 
{
    Runtime theSystem = Runtime.getRuntime();
    Vector depnames = new Vector();		    
    String downloaddir = Constants.DEMODIR + "downloads";	
    String rpmlist = "";
    boolean downloadsuccess = true;
    Vector downloadedFileNames = new Vector();

    int roundCounter = 1;
    boolean fetchSuccess = false;
    
    // Open a new file for dependencies list 
    BufferedReader depbuf; 

    // Initialize the system runtime and do a test install
    public FetchDependencies(String packagenames)
    {	
	rpmlist = packagenames;
    }

    // Method to return if the download dependencies was a success or not
    public boolean isSuccess()
    {
	return fetchSuccess;
    }

    // Method to try a test installation to get the list of failed dependencies if any
    // This method takes a single package name or a space separated list of packagenames , which is just a single argument
    public void testInstall()
    {
	try
	{
	    // Test installation command 
	    String execStr = "rpm -i --test " + rpmlist;
	    //System.out.println(execStr);
	    Process proc = theSystem.exec(execStr);
	    proc.waitFor();
	    int val = proc.exitValue();
	    
	    // After the process exec returns (finishes) along with an error message 
	    if(val > 0)
	    {
		BufferedReader buf = new BufferedReader(new InputStreamReader(proc.getErrorStream()));		
		
		// Open a new file for writing the list of dependencies and their corresponding versions , if any 
		BufferedWriter outbuf = new BufferedWriter(new FileWriter("/tmp/dependencies.txt"));		    
		//StringTokenizer strtok ;
		String s;     
		String pkgname = "" ,version = "";
		int indexofneeded;
		int indexofequals;
		
		while ((s = buf.readLine()) != null)			
		{
		    // remove off the trailing or leading spaces , if any 
		    s = s.trim();
		    
		    // In case of failed dependencies , get the dependency name
		    indexofneeded = s.indexOf("is needed by");
		    if (!(s.startsWith("error:")) && (indexofneeded >= 0))
		    {			    			    
			pkgname = s.substring(0,s.indexOf(' '));		
			
			// In case a particular failed dependency contains info about a required version , write the name along with the version to file 
			indexofequals = s.indexOf(">=");
			if(indexofequals >= 0)
			{				
			    version = s.substring(indexofequals,indexofneeded - 1);
			    version = version.substring(version.indexOf(" ") + 1);
			    if ((depnames.indexOf(pkgname + ";" +  version) < 0))
			    {
				depnames.addElement(pkgname + ";" + version);
			    }
			}
			// In case of no version info , write only the name to a file 
			else
			{
			    if (depnames.indexOf(pkgname) < 0)
			    {				    				
				depnames.addElement(pkgname);
			    }
			}
		    }
		} // end of while
		
		// This is just for test printing and can be removed later
		GUI.jlCurrentAction.setText("Downloading required dependency packages..........");
		System.out.println("\nThe failed dependencies are : "); 
		for(int i = 0 ; i < depnames.size(); i++)
		{			
		    System.out.println(depnames.elementAt(i));
		    outbuf.write((String)depnames.elementAt(i) + "\n");			
		}		
		buf.close();
		outbuf.close();
		System.out.println("\n");		
		if (depnames.size() > 0)
		{
		    depnames.clear();

		    if (roundCounter == 1)
		    {
			String title = "Downloading package(s)";
			String message = "This installation will need to download and install some package(s)\n" + 
			                 "in order to complete . Do you wish to proceed ?";

			Object options[] = {"Yes", "No"};		    
			GUI.pauseProgressBar();	
			int n = JOptionPane.showOptionDialog(null,message ,title ,JOptionPane.YES_NO_OPTION ,
							     JOptionPane.INFORMATION_MESSAGE, null, options, options[0]);
			
			// Abort installation 
			if (n == JOptionPane.NO_OPTION)
			{		
			    GUI.stopProgressBar();
			    return;
			}
			// Show details of violations
			else if (n == JOptionPane.YES_OPTION)
			{
			    GUI.resumeProgressBar();
			}
		    }
		    //proceed = true;
		    // Open the list of dependencies now as we need to get the corresponding URLs for each dependency
		    if (fetchURLs())
			// Open the list of download urls to initiate the downloads
			getURLsList();
		}
		else
		    fetchSuccess = true;
		
	    }  // end of   if (val > 0)
	    
	    // The process returned without an error message , so the test install was successful and there are no failed dependencies
	    else
	    {
		System.out.println("Round : " + roundCounter); 
		System.out.println("Package Test install successful \n");
		fetchSuccess = true;
	    }
	}  // end of try
	catch(InterruptedException e) 
	{
	    System.out.println("The process for the test install was interrupted.");
	    System.out.println(e.getMessage());
	}
	catch(IOException e)
	{
	    System.out.println("The process encountered an IO exception.");
	    System.out.println(e.getMessage());
	}
	catch(Exception e) 
	{
	    System.out.println("There was a problem in getting the Runtime Env.");
	    System.out.println(e.getMessage());
	}
    }

    // This method will actually open the dependencies list and use perl or some other language to get the list of URLs for 
    // that dependency. This list will then be written to a file for the corresponding dependency
    public boolean fetchURLs()
    {
	try
	{
	    depbuf = new BufferedReader(new FileReader("/tmp/dependencies.txt"));		
	    String depname  = "";
	    
	    while ((depname = depbuf.readLine()) != null)
	    {
		int indexofversion = depname.indexOf(";");
		if (indexofversion >= 0)
		    depname = depname.substring(0,indexofversion);
		
		//String url =  "\"http://www.rpmfind.net/linux/rpm2html/search.php?query=" + depname + "&submit=Search+...&system=redhat&arch=i386\"";
		//String  execStr = "wget -O " + depname + ".html " + url; 
		//System.out.println(execStr);

		// line commented for demo purpose
		String htmlname = "/tmp/" + depname + ".html";
		String [] execStrArray = {"wget", "-O", htmlname, "http://www.rpmfind.net/linux/rpm2html/search.php?query=" + depname + "&submit=Search+...&system=redhat-" + Constants.REDHATVERSION + "&arch=i386"};

		//String [] execStrArray = {"wget","-O",depname + ".html","http://129.49.0.1/demo/html/" + depname + ".html"};

		for (int i = 0 ; i < execStrArray.length; i++)
		    System.out.println(execStrArray[i]);
		Process proc = theSystem.exec(execStrArray);
		proc.waitFor();
		int val = proc.exitValue();
		System.out.println("Val for wget is : " + val);
		// If the process exec returns (finishes) along with an error message 
		if(val > 0)
		{
		    System.out.println("Error in extracting the HTML file for  " + depname + "  rpm extraction");
		    BufferedReader buf = new BufferedReader(new InputStreamReader(proc.getErrorStream()));	
		    String s;
		    while ((s = buf.readLine())!= null)
			System.out.println(s);
		    buf.close();
		}
		else
		{
		    System.out.println("HTML file download successful for  " + depname + "  rpm extraction");
		    String execStr = "perl tools/parsehtml.pl " + htmlname + " " + "/tmp/downloadurls.txt"; 
		    System.out.println(execStr);
		    proc = theSystem.exec(execStr);
 		    proc.waitFor();
		    val = proc.exitValue();

		    // If the process exec returns (finishes) along with an error message 
		    if(val > 0)
			System.out.println("Error in parsing the HTML file for : " + depname);
		    else
		    {
			new File(Constants.DEMODIR + depname + ".html").delete();
			System.out.println("HTML file parsing successful for : " + depname + " . HTML file deleted");
		    }		    
		}
	    }
	}
	catch(FileNotFoundException e)
	{
	    System.out.println("File Not found Exception in fetchURLs : " + e.getMessage());
	}
	catch(IOException e) 
	{
	    String errmsg = e.getMessage();
	    if ((errmsg.indexOf("wget: not found") >= 0)) 
	    {
		System.out.println("wget not installed");
	        GUI.pauseProgressBar();
		JOptionPane.showMessageDialog(null,"Wget is not installed.\nVersion 1.6 or above expected for automatic downloads to work.","Test Installation Failure", JOptionPane.INFORMATION_MESSAGE);
		GUI.resumeProgressBar();
	    }
	    else if ((errmsg.indexOf("perl: not found") >= 0)) 
	    {
		System.out.println("perl not installed");
	        GUI.pauseProgressBar();
		JOptionPane.showMessageDialog(null,"perl is not installed.\nVersion 5.6.0-12 or above expected for automatic downloads to work.","Test Installation Failure", JOptionPane.INFORMATION_MESSAGE);
		GUI.resumeProgressBar();
	    }
	    System.out.println("IOException in fetchURLs : " +e.getMessage());
	    return false;
	}
	catch(Exception e)
	{
	    System.out.println("Exception encountered in fetchURLs " + e.getMessage());
	}	    
	return true;
    }

    // Method to open up the download urls list & get the corresponding URLS so that the dependencies can be downloaded
    public void getURLsList()
    {
	try
	{
	    // Open up the file containing the list of the download urls for various dependencies
	    BufferedReader downldbuf = new BufferedReader(new FileReader("/tmp/downloadurls.txt"));
	    depbuf    = new BufferedReader(new FileReader("/tmp/dependencies.txt"));		

	    String url = "";	    
	    String packagedownld = "";	
    	    String nextpackage = "";	
	    Vector listurls = new Vector();	
	    int flag = 0 ;   

	    while ((url = downldbuf.readLine()) != null)
	    {	
		// For every subsequent dependency name after the first dependency
		if ((url.startsWith("(")) && (flag == 1))
		{
		    nextpackage = url.substring(1,url.length() - 1);
		    if (!packagedownld.equals(""))
		    {
			// sort URLs and prioritize giving the package name and the list of urls
			 selectURLs(packagedownld,listurls);
		    }
		    listurls.clear();
		    packagedownld = nextpackage;
		    System.out.println("##################   Package name : " + packagedownld + "   ##################");
		    continue;
		} 

		// Comes here when it enters the loop for the first time
		if (url.startsWith("("))
		{
		    packagedownld = url.substring(1,url.length() - 1);
		    System.out.println("##################   Package name : " + packagedownld + "   ##################");
		    flag = 1;
		    continue;
		}

		// Add the list of URLs for the current dependency to a vector
		listurls.addElement(url);
		//System.out.println(listurls.elementAt(listurls.size() - 1));		
	    }

	    // After all the dependencies are taken care of , the last one still needs to be catered to
	    if (!packagedownld.equals(""))
	    {
		// sort URLs and prioritize giving the package name and the list of urls
		selectURLs(packagedownld,listurls);			    	    
	    }
	    depbuf.close();
	    downldbuf.close();
	    if (downloadsuccess)
	    {		
		//System.out.println("The download process was successful . Trying a test install again ....\n");
	    }
	    else
	    {
		System.out.println("The download process was unsuccessful \n");
	    }

	    System.out.println("###################################  End of Round  " + roundCounter + "  #################################");	    

	    // Here we should try a test install again as the downloading process has ended
	    roundCounter++;
	    new File("/tmp/downloadurls.txt").delete();	    
	    new File("/tmp/dependencies.txt").delete();

	    String newlist = "";
	    File downloaddir = new File(Constants.DEMODIR + "downloads");
	    File [] list = downloaddir.listFiles();
	    for (int i = 0 ; i < list.length ; i++)
		newlist = newlist + " " + list[i].getAbsolutePath();

	    rpmlist = rpmlist + " " + newlist;
	    //System.out.println("The new list of rpms : " + rpmlist);
	    if (roundCounter <= 5)
	    {
		testInstall();
	    }
	    else
	    {
		String message = "Some of the packages you selected have many unresolved dependencies .\n" +
                                 "It is advisable to abort the current installation and try deselecting \n" +
             	 	         "packages so that the installation can continue. Alternatively, please \n" + 
                                 "download the required packages and then install the currently selected packages.\n\n";

		String title   = "RPM Downloads : Dependency problem";
	
		GUI.pauseProgressBar();
		JOptionPane.showMessageDialog(null, message, title, JOptionPane.INFORMATION_MESSAGE);
		roundCounter = 1;
		GUI.stopProgressBar();
	    }
	}
	catch(FileNotFoundException e)
	{
	    System.out.println("File not found");
	    System.out.println(e.getMessage());
	}
	catch(IOException e) 
	{
	    System.out.println("File reading exception");
	    System.out.println(e.getMessage());
	}
	catch(Exception e)
	{
	    System.out.println("Exception encountered ");
	    System.out.println(e.getMessage());
	}	    
    }

    
    // Method to select and prioritize the URLs from the URLs obtained for a particular dependency
    public void selectURLs(String packagename,Vector listofurls )
    {	
	boolean success = false;
	try
	{	    	    
	    // This string will be used for reading every line of the dependencies list 
	    String dep;
	    
	    // The name of the failed dependency package 
	    String packagedep;
	    // The package version 
	    String version;	    
	    Vector newlistofurls = new Vector();	    

	    String filename; 	
	    int indexofversion;
	
	    // Read the corresponding dependency name and version if any from the dependencies file
	    if ((dep = depbuf.readLine()) != null)
	    {
		// get the index of the semicolon before the version number , if any 
		indexofversion = dep.indexOf(";");		    
		
		// if there is a version number given
		if(indexofversion >= 0)
		{
		    version = dep.substring(indexofversion + 1);   // get the version 
		    packagedep = dep.substring(0,indexofversion);  // get the package name 		    
		}
		// version is not given , so download the dependency without sorting 
		else
		{
		    version = "";       // no version specified , so any url would do 
		    packagedep = dep;   // get the package name 
	        }	
		
		//System.out.println("Dependency name : " +packagedep + " Download name : " + packagename); 
		if(packagedep.equals(packagename))
		{
		    if(!version.equals(""))
		    {
			// Since  a version choice exists , sort the urls before download
			newlistofurls = CommonMethods.sortURLs(listofurls,version);		    
			System.out.println("The sorted list of URLS for package  " + packagename +  "  is : ");
			for (int i = 0; i < newlistofurls.size() ; i++)
			{
			    System.out.println(newlistofurls.elementAt(i));
			}
			// download the dependency trying to match the version number
			success = downloadDependencies(newlistofurls); 
		    }
		    else
		    {
			// download the dependency straightaway as no version number is given
			success = downloadDependencies(listofurls);
		    }
		}
		else
		{
		    System.out.println("\nError in reading dependencies file ....Package name mismatch");
		}
	    }
	    // no data could be read from the buffer
	    else 
	    {
		System.out.println("\nEnd of dependencies list . Error in reading dependencies file ....");
	    }
		
            // if any of the downloads report failure , this means that the download process has failed and 
	    // there is a good chance that the software installation would not go through 
	    if(success)
		downloadsuccess = true;
	    else
		downloadsuccess = false;
	}
	catch(FileNotFoundException e)
	{
	    System.out.println("File not found");
	    System.out.println(e.getMessage());
	}
	catch(IOException e) 
	{
	    System.out.println("File reading exception");
	    System.out.println(e.getMessage());
	}
	catch(Exception e)
	{
	    System.out.println("Exception encountered ");
	    System.out.println(e.getMessage());
	}	 
    }

    // Method to actually download the dependency files when passed a list of the corresponding urls
    // returns : a success flag indicating the result of the download
    public boolean downloadDependencies(Vector listofurls)
    {
	String url;
	String filename;
	boolean success = true;

	//System.out.println("\nDownload destination : " + downloaddir + "\n");

	// Iterate thru the list of urls for a particular dependency till one of them is successfully downloaded 
	for(int i = 0; i < listofurls.size() ; i++)
	{
	    // get the string associated with the URL
	    url = (String) listofurls.elementAt(i);

	    // get only the name of the file to be downloaded
	    filename = CommonMethods.getFileName(url);	

	    System.out.println("-------------------------------------------------------");
	    System.out.println("\nDownloading file :  " + filename + "  from  " + url);
	    
	    try
	    {
		String execStr = "wget -nc -P " + downloaddir + " " + url;  		    
		System.out.println(execStr);
		Process proc = theSystem.exec(execStr);
		proc.waitFor();
		int val = proc.exitValue();
		
		// If there is any error during wget exec call
		if(val > 0)
		{
		    try
		    {
			success = false;
			BufferedReader errorbuf = new BufferedReader(new InputStreamReader(proc.getErrorStream()));		
			String s;			
			
			// Error encountered during download
			while ((s = errorbuf.readLine()) != null)
			{
			    //System.out.println(s);     //  printing out the error message line by line
			}
			// close the error stream read buffer 
			errorbuf.close();        			
		    }
		    catch(IOException e) 
		    {
			System.out.println("The process for downloading the file was interrupted.");
			System.out.println(e.getMessage());
		    }
		    catch(Exception e)
		    {
			System.out.println("The process encountered an exception.");
			System.out.println(e.getMessage());
		    }
		}
		// The download was successful so come out of the loop
		else
		{
		    success = true;
		    // Add the downloaded filename to a vector if the file is successfully downloaded
		    if (downloadedFileNames.indexOf(filename) < 0)
			downloadedFileNames.add(filename);
		    break;
		}
	    }  // end of outer try	    
	    catch(IOException e) 
	    {
		success = false;
		System.out.println("The process for downloading the file was interrupted.");
		System.out.println(e.getMessage());
	    }
	    catch(Exception e)
	    {
		success = false;
		System.out.println("The process encountered an exception.");
		System.out.println(e.getMessage());
	    }
	    finally
	    {
		if (success)
		    System.out.println("File  " + filename  + "  downloaded successfully !!!!\n");
		else
		    System.out.println("File  " + filename  + "  could not be downloaded !!!!\n"); 		
	    }
	} // end of for loop
	return success;
    }

    public static void main(String[] args)
    {
	String packageList = "";
	for(int i = 0; i < args.length ; i++)
	{
	    packageList = packageList + " " + args[i];
	}
	FetchDependencies objFD = new FetchDependencies(packageList);
	objFD.testInstall();
	if (objFD.isSuccess())
	    System.out.println("The Test Install was successful and the installation can proceed ");
	else
	    System.out.println("The Test Install failed . Installation cannot proceed ");
	System.exit(0);
    }
}
    

