

/*  Copyright (C) 2002 Secure Systems Lab, Stony Brook. 

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program (See file COPYING in the main directory); 
    if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*   Written by Tapan Kamat tkamat@cs.sunysb.edu */

import javax.swing.*;
import javax.swing.tree.*;
import java.io.File;

public class FileNode extends DefaultMutableTreeNode 
{ 
    protected boolean         explored = false;
    protected CheckableObject chkobj;

    // Constructor 
    public FileNode(CheckableObject chkobj)
    {
	super (chkobj);
	this.chkobj = chkobj;
    }

    // returns the reference to the inner checkable object
    public void setChecked(boolean value)
    {
	chkobj.setChecked(value);
    }

    // gets the value of the explored flag
    public boolean isExplored() 
    { 
	return explored; 
    }

    // sets the value of the explored flag
    public void setExplored(boolean value) 
    { 
	explored = value; 
    }

    // This method expands a node given the list of the node's children
    public void expandNode(File [] children)
    {
	for (int i = 0; i < children.length; i++) 
	{
	    // get a node handle for each child
	    FileNode child = new FileNode(new CheckableObject(children[i]));

	    // if child is a directory add a dummy
	    if (children[i].isDirectory()) 
	    {
		child.setAllowsChildren(true);
		FileNode dummy = new FileNode(new CheckableObject(new File("")));
		child.add(dummy);
	    }
	    // if child is a file , make it a leaf
	    else 
	    {
		child.setAllowsChildren(false);
	    }
	    child.explored = true;
	    // add child to node to be expanded
	    add(child);
	}
    }
}
