

/*  Copyright (C) 2002 Secure Systems Lab, Stony Brook. 

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program (See file COPYING in the main directory); 
    if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*   Written by Tapan Kamat tkamat@cs.sunysb.edu */

import java.lang.*;
import java.io.*;
import java.util.*;
import javax.swing.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */


// This class fills the owned and shared vectors according to the package which is about to be installed
public class FindPackage 
{
    Runtime theSystem = Runtime.getRuntime();

    Vector createNotOwnViolations = new Vector();
    Vector writeOwnModViolations = new Vector();
    Vector writeNotOwnViolations = new Vector();
    Vector writeSharedViolations = new Vector();    

    String summaryViolations = "";
    String detailedViolations = "";
	
    Vector ownedDirs = new Vector();    
    String currentPackagePath = "";
    String currentPackageName = "";
    String currentPartialPackageName = "";

    int installationType;              // 0 : default , 1: existing , 2: upgrade , 3: new installation 
    int currentIndex;

    public FindPackage(String path) 
    {
	currentPackagePath = path;
    }


    // This method gets the Installation type for every package that is about to be installed 
    // Installation type : Existing , Upgrade or New 
    public void getInstallationType()
    {
	Vector name  = CommonMethods.getFullPackageName(currentPackagePath);
	currentPackageName = (String) name.elementAt(0);
	System.out.println("\nPackage name that will be installed by file  : " + currentPackageName);

	name = CommonMethods.getPartialPackageName(currentPackagePath);
	currentPartialPackageName = (String) name.elementAt(0);

	currentIndex = GUI.currentIndex;

	Vector installedList = CommonMethods.queryRPMDatabase(currentPartialPackageName);
	String installedPackage = (String) installedList.elementAt(0);

	// New installation
	if (installedPackage.endsWith("is not installed"))
	{
	    System.out.println("New installation\n");
	    GUI.summaryReport [currentIndex] += "\n\nInstallation type : New installation";
	    GUI.detailedReport[currentIndex] += "\n\nInstallation type : This package installation does not exist on the system";			
	    installationType = 3;
	}
	// The specified package is already installed	
	else if (installedPackage.equals(currentPackageName)) 
	{
	    System.out.println("The specified exact version of this package is already installed .\n");
	    GUI.summaryReport [currentIndex] += "\n\nInstallation type : Already installed";
	    GUI.detailedReport[currentIndex] += "\n\nInstallation type : No action will be taken as this package is already installed";
	    installationType = 1;
	}
	// Upgrade of a particular installation
	else
	{ 
	    System.out.println("Another version of this package is installed. Upgrade required\n");
	    GUI.summaryReport [currentIndex] += "\n\nInstallation type : Upgrade installation";
	    GUI.detailedReport[currentIndex] += "\n\nInstallation type : Another version of this package exists.Upgrade will be performed";
	    installationType = 2;
	}
    }

    // this method gets the list of directories owned by a package and populates them into the ownedDirectories vector
    public void getOwnedDirectories()
    {
	try
	{
	    String execStrArray[] = {"rpm", "-qp", "--queryformat", "[%{DIRNAMES}\n]", currentPackagePath};
	    //System.out.println(execStrArray);
	    Process proc = theSystem.exec(execStrArray);
	    proc.waitFor();
	    int val = proc.exitValue();
	    
	    // Normal Termination (We get the result on the output stream)
	    if (val == 0) 
	    {
		// This always sends message to the standard output , hence getInputStream is used
		BufferedReader buf = new BufferedReader(new InputStreamReader(proc.getInputStream()));
		String s;
		try
		{
		    while((s = buf.readLine()) != null)
		    {
			if ( (s.length() > 1) && (s.endsWith("/")) )
			    s = s.substring(0,s.length() - 1);
			//System.out.println(s);
			
			// if this dir name is not present in the list of owned directories(sometimes there are duplicates)
			if (ownedDirs.indexOf(s) < 0)
			{
			    File dir = new File(s);

			    // if directory is owned ,populate the list of owned directories
			    if ( !dir.exists() ||  isOwnedDirectory(s))
				ownedDirs.add(s);		    
			}
		    }
		    System.out.println("\n\nList of directories exclusively owned by this package : ");
		    for (int i = 0; i < ownedDirs.size() ; i++)
			System.out.println(ownedDirs.elementAt(i));
		    System.out.println("\n\n");
		}
		catch(IOException e)
		{
		    System.out.println("IO Exception in readLine : " + e.getMessage());
		}
		buf.close();
	    }
	}
	catch(InterruptedException e)
	{
	    System.out.println("Interrupted Exception in proc.waitFor in method getOwnedDirectories : "  + e.getMessage());
	}
	catch(Exception e)
	{
	    System.out.println("Exception in method getOwnedDirectories: "  + e.getMessage()); 
	}				
    }

    private boolean isOwnedDirectory(String dirname)
    {
	// If the directory name is in the list of shared directories,we return false
	if (GUI.sharedDirectories.contains(dirname))
	{
	    System.out.println(dirname + " is a shared directory");
	    return false;
	}

	// If the filename is not a name of a directory , then we return false
	if (!(new File(dirname)).isDirectory())
	    return false;

	// get the owner of this directory
	Vector owners = CommonMethods.getOwner(dirname);

	// If number of owners is greater than 1 , then the directory is not owned
	if (owners.size() > 1)
	    return false;
	// If number of owners is 1 , then the directory could be owned by the package itself
	else if (owners.size() == 1) 
	{
	    if (currentPartialPackageName.equals((String)owners.elementAt(0)))
		return true;
	    else 
		return false;
	}
	// Check the directory contents if this directory has no owners
	else
	{
	    File dir = new File(dirname);
	    File[] files = dir.listFiles();
	    for(int i = 0; i < files.length ; i++)
	    {
		String filename = files[i].getAbsolutePath();
		//System.out.println(filename);  

		// Check if this directories is in the list of shared directories
		if (GUI.sharedDirectories.contains(filename))
		{
		    System.out.println(filename + " is a shared directory");
		    return false;
		}

		owners = CommonMethods.getOwner(filename);
		// If number of owners is greater than 1 , then the directory is not owned
		if (owners.size() > 1)
		    return false;
		// If number of owners is 1, then the directory could be owned by the package itself
		else if (owners.size() == 1) 
		{
		    if (!currentPartialPackageName.equals((String)owners.elementAt(0)))
			return false;
		}
	    }		
	}
	return true;
    }

    // This method categorizes the list of files to be installed by the package into Owned, Shared or Changed category 
    public  void categorizeFiles() 
    {
        try
        {
	    BufferedWriter creatable = new BufferedWriter(new FileWriter("/tmp/" +CommonMethods.getPackageNameWithVersion(currentPackagePath)+ "-create"));
	    BufferedWriter writable = new BufferedWriter(new FileWriter("/tmp/" +CommonMethods.getPackageNameWithVersion(currentPackagePath)+ "-write"));
	    BufferedWriter readable = new BufferedWriter(new FileWriter("/tmp/" +CommonMethods.getPackageNameWithVersion(currentPackagePath)+ "-read"));

	    String execStr = "rpm - qpl " + currentPackagePath;
	    //System.out.println(execStr);
	    Process proc = theSystem.exec(execStr);
	    proc.waitFor();
	    int val = proc.exitValue();
	    
	    // Normal Termination (We get the result on the output stream)
	    if (val == 0) 
	    {
		// This always sends message to the standard output , hence getInputStream is used
		BufferedReader buf = new BufferedReader(new InputStreamReader(proc.getInputStream()));
		String s;
		String backupFilesList = "";
		while ((s = buf.readLine()) != null)
		{
		    // File has been modified after installation
		    boolean writeAllowed = true;
		    File pkgFile = new File(s);

		    // File being installed does not exist, hence has to be created
		    if (!pkgFile.exists())
		    {
			// The following create is allowed
			if (GUI.jcbCreateNotOwn.isSelected())
			    creatable.write(s + "\n");
			// Create in not owned directory is not allowed
			else
			{
			    // File is being installed into an exclusively owned directory
			    if (isOwned(s))
				creatable.write(s + "\n");
			    else
			    {
				createNotOwnViolations.add(s);
				System.out.println("Create : Not owned    Summary violation for : " + s);
			    }
			}
		    }
		    // File being installed already exists, hence is being overwritten
		    else
		    {
			// The following writes are allowed , so we need not check for violations except modifications post installation
			if (GUI.jcbWriteOwnMod.isSelected() && GUI.jcbWriteNotOwn.isSelected()) 
			{
			    // File has been modified post installation 
			    if(CommonMethods.hasChanged(s))
				backupFilesList += " " + s;
			}
			// One of the writes is not allowed so check thoroughly
			else
			{
			    // File has been modified post installation 
			    if(CommonMethods.hasChanged(s))
			    { 	
				backupFilesList += " " + s;
				
				// Writes to owned but modified files are disallowed
				if (!GUI.jcbWriteOwnMod.isSelected()) 
				{
				    writeAllowed = false;
				    writeOwnModViolations.add(s);
				    System.out.println("Write : Owned but modified    violation for : " + s);
				}
			    }
			    // Write to file in not owned directory is disallowed
			    else if (!GUI.jcbWriteNotOwn.isSelected())
			    {
				if (!isOwned(s))
				{
				    writeAllowed = false;
				    writeNotOwnViolations.add(s);
				    System.out.println("Write : Not owned directory   violation for : " + s);
				}
			    }
			}
		    }

		    if (pkgFile.exists())
		    {
			// Write to shared files is allowed
			if (GUI.jcbWriteShared.isSelected())
			{
			    if (writeAllowed)
				writable.write(s + "\n");
			}
			// Write to shared files is not allowed
			else
			{
			    // File being installed is shared by multiple packages
			    if (isSharedFile(s))
			    {
				writeSharedViolations.add(s);
				System.out.println("Write : Shared file violation for : " + s);
			    }
			    // Not a shared file
			    else if (writeAllowed)
				writable.write(s + "\n");
			}
		    }
		    readable.write(s + "\n");

		} // end of while loop

		// These are file which need to be written to during rpm installations, so add the corresponding entry to /tmp/write
		writable.write("/var/lib/rpm/Basenames" + "\n");
		writable.write("/var/lib/rpm/Conflictname" + "\n");
		writable.write("/var/lib/rpm/Group" + "\n");
		writable.write("/var/lib/rpm/Name" + "\n");
		writable.write("/var/lib/rpm/Packages" + "\n");
		writable.write("/var/lib/rpm/Providename" + "\n");
		writable.write("/var/lib/rpm/Requirename" + "\n");
		writable.write("/var/lib/rpm/Triggername" + "\n");
		writable.write("/dev/null" + "\n");

		// close all file buffers
		creatable.close();
		writable.close();
		readable.close();

		System.out.println("Total createNotOwnViolations    : " + createNotOwnViolations.size());
		System.out.println("Total writeOwnModViolations     : " + writeOwnModViolations.size());
		System.out.println("Total writeNotOwnViolations     : " + writeNotOwnViolations.size());
		System.out.println("Total writeSharedViolations     : " + writeSharedViolations.size());

		if (!backupFilesList.equals(""))
		    backupFiles(backupFilesList);
	    }
	}
	catch(InterruptedException e)
	{
	    System.out.println("Interrupted Exception in proc.waitFor in method  categorizeFiles : "  + e.getMessage());
	}
	catch(IOException e)
	{
	    System.out.println("IOException Exception in method  categorizeFiles  : "  + e.getMessage());
	}
	catch(Exception e)
	{
	    System.out.println("Exception in method  categorizeFiles : "  + e.getMessage()); 
	}
    }


    // This method will find out if a particular file is being installed into an exclusively owned directory or in a shared directory
    private boolean isOwned(String filename)
    {
	//System.out.println("File/ Dir name : " + filename);
	File packageFile = new File(filename);

	// this is assumed to be a directory name so check for it in the list of owned dirs	
	if (packageFile.isDirectory())
	{
	    if (ownedDirs.contains(filename))
	    {
		//System.out.println("\nThis directory : " + filename + "    is owned by the package\n");
		return true;
	    }
        }
	// this is a file so get its parent directory and check for the dir in the list of owned dirs
	else
	{
	    String parentName = packageFile.getParent();
	    //System.out.println("File name : " + filename + "     Parent name : " + parentName);
	    if (ownedDirs.contains(parentName))
	    {
		//System.out.println("This file will be installed in an owned directory\n");
		return true;
	    }
	}	    
	return false;
    }

    private boolean isSharedFile(String filename)
    {
	int i = 0;
	try
	{
	    String execStr = "rpm -qf " + filename;
	    Process proc = theSystem.exec(execStr);
	    proc.waitFor();
	    int val = proc.exitValue();
	    BufferedReader buf = new BufferedReader(new InputStreamReader(proc.getInputStream()));
	    try
	    {
		while(buf.readLine()!= null)
		{
		    if (++i > 1)
			break;
		}    		   
	    }
	    catch(IOException e)
	    {
		System.out.println("IO Exception in readline : " + e.getMessage());
	    }
	    buf.close();
	}
	catch(InterruptedException e)
	{
	    System.out.println("Interrupted Exception in proc.waitFor in getFullPackageName: "  + e.getMessage());
	}
	catch(Exception e)
	{
	    System.out.println("Exception in method getFullPackageName : "  + e.getMessage()); 
	}	
	finally
	{
	    if (i > 1)
		return true;
	    else
		return false;
	}
    }

    //backup the sharedFiles into a tarball with the same package name 
    private boolean backupFiles(String backupFilesList)
    {	
	boolean success = false;
	try
	{
	    Runtime theSystem = Runtime.getRuntime();
	    String execStr = "tar -zcvf " + "/tmp/bkp-" +CommonMethods.getPackageNameWithVersion(currentPackagePath)+ ".tgz -P " + backupFilesList;
	    System.out.println("Exec string : "  + execStr);
	    Process proc = theSystem.exec(execStr);	    
	    proc.waitFor();
	    int val = proc.exitValue();

	    // Normal termination
	    if (val == 0)
	    {
		System.out.println("Normal Termination for backupFiles");
		BufferedReader buf = new BufferedReader(new InputStreamReader(proc.getInputStream()));
		String s; 
		while ((s = buf.readLine()) != null) 
		{
		    System.out.println(s);
		}
		success = true;
		buf.close();
	    }	    
            else
	    {
		System.out.println("Abnormal Termination for backupFiles");
		success = false;
            }
	}
	catch(InterruptedException e)
	{
	    System.out.println("Interrupted Exception in proc.waitFor in method backupFiles : "  + e.getMessage());
	}
	catch(IOException e)
	{
	    System.out.println("IOException Exception in method backupFiles : "  + e.getMessage());
	}
	catch(Exception e)
	{
	    System.out.println("Exception in method backupFiles : "  + e.getMessage()); 
	}
	finally
	{
	    return success;
	}
    }
    
    // Method which writes to the report file
    public void writeToReport()
    {
	GUI.summaryReport[currentIndex] = GUI.summaryReport[currentIndex] + "\n\nFile Analysis";
	GUI.summaryReport[currentIndex] = GUI.summaryReport[currentIndex] + "\n---------------\n";
	GUI.summaryReport[currentIndex] = GUI.summaryReport[currentIndex] + "\nViolating File Names  :\n\n";
	
	GUI.detailedReport[currentIndex] = GUI.detailedReport[currentIndex] + "\n\nFile Analysis";
	GUI.detailedReport[currentIndex] = GUI.detailedReport[currentIndex] + "\n---------------\n";
	
	if (anyFileViolations())
	{
	    System.out.println("Some file violation found, so it must be reported");
	    if (createNotOwnViolations.size() > 0)
	    {
		detailedViolations  += "\nCreated in Not Owned Directory";		    
		detailedViolations  += "\n------------------------------\n";  
		writeViolations(createNotOwnViolations);
	    }
	    if (writeOwnModViolations.size() > 0)
	    {
		detailedViolations  += "\nFiles modified after installation";		    
		detailedViolations  += "\n---------------------------------\n";  
		writeViolations(writeOwnModViolations);
	    }
	    if (writeNotOwnViolations.size() > 0)
	    {
		detailedViolations  += "\nWritten in Not Owned Directory";		    
		detailedViolations  += "\n------------------------------\n";  
		writeViolations(writeNotOwnViolations);
	    }
	    if (writeSharedViolations.size() > 0)
	    {
		detailedViolations  += "\nShared files written";		    
		detailedViolations  += "\n--------------------\n";  			
		String name = "";
		for (int i = 0; i < writeSharedViolations.size() ; i++)
		{
		    name = (String) writeSharedViolations.elementAt(i);
		    // This filename has not already been included in the previous 3 cases
		    if ((createNotOwnViolations.indexOf(name) < 0)
			|| (writeOwnModViolations.indexOf(name) < 0)
			|| (writeNotOwnViolations.indexOf(name) < 0))
		    {
			summaryViolations = summaryViolations +  name + "\n";
		    }
		    
		    detailedViolations = detailedViolations + name + "\n";  			
		}	    
	    }
	    GUI.summaryReport[currentIndex] = GUI.summaryReport[currentIndex] + summaryViolations;
	    GUI.detailedReport[currentIndex] = GUI.detailedReport[currentIndex] + detailedViolations; 
	}
	else
	{
	    System.out.println("No file violation found\n");
	    GUI.summaryReport[currentIndex] = GUI.summaryReport[currentIndex] + "\nNo violation detected\n";
	    GUI.detailedReport[currentIndex] = GUI.detailedReport[currentIndex] + "\nNo violation detected\n"; 
	}		
    }


    // Method which returns true if there are any violations
    private boolean anyFileViolations()
    {
	if  ((createNotOwnViolations.size() > 0)
	     || (writeOwnModViolations.size() > 0)	
	     || (writeNotOwnViolations.size() > 0) 
	     || (writeSharedViolations.size() > 0))
	    return true;
	return false;
    }

    // Method to write violation names to the report
    private void writeViolations(Vector filelist)
    {
	String s = "";
	for (int i = 0; i < filelist.size() ; i++)
	{
	    s = (String) filelist.elementAt(i);
	    summaryViolations = summaryViolations +  s + "\n";
	    detailedViolations = detailedViolations + s + "\n";  			
	}	    
    }


} //end class






































































