

/*  Copyright (C) 2002 Secure Systems Lab, Stony Brook. 

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program (See file COPYING in the main directory); 
    if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*   Written by Tapan Kamat tkamat@cs.sunysb.edu */

import java.lang.*;
import java.io.*;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.*;
import javax.swing.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

public class FindScripts 
{
    Runtime theSystem = Runtime.getRuntime();
    String currentPackage;
    String currentPackagePath;	

    String preInstallFile; 
    String postInstallFile;

    String preInstallScript = "";
    String postInstallScript = "";

    static String scriptType = "";
    boolean isViolating = false;
    String summaryViolations = "";
    String detailedViolations = "";
    int currentIndex;
    
    public FindScripts(String strFilename) 
    {
	File filename = new File(strFilename);
	currentPackage = CommonMethods.getPackageNameWithVersion(strFilename);
	preInstallFile = Constants.DEMODIR + "scripts/" + currentPackage + ".preinstall.sh";
	postInstallFile = Constants.DEMODIR + "scripts/" + currentPackage + ".postinstall.sh";
	currentPackagePath = strFilename;
	currentIndex = GUI.currentIndex;
    }

    public void getScripts(String scriptType)
    {
	try 
	{
	    String type;
	    String filename;
	    if (scriptType.equals("preinstall"))
	    {
		type = "%{PREIN}";
		filename = preInstallFile;
	    }
	    else
	    {
		type = "%{POSTIN}";
		filename = postInstallFile;
	    }

	    //System.out.println("Type : " + type +"\tFile : " + filename);
	    // Note : It is necessary to add double quotes to the ends of this string and once again do the same
	    //        otherwise this command fails
	    String command = "\"rpm -qp --queryformat " + type + " " + currentPackagePath + " | " + "grep -v none " + " > " + filename + "\"";
	    //System.out.println("Command : " +  command);
	    String execStrArray[] = {"/bin/sh", "-c", "\"" + command + "\""};
	    Process proc = theSystem.exec(execStrArray);
	    proc.waitFor();
	    int val = proc.exitValue();
	    
	    // Abnormal Termination (We get the result on the error stream)
	    if (val != 0) 
	    { 
		BufferedReader buf = new BufferedReader(new InputStreamReader(proc.getErrorStream()));
		String s;		
		try
		{
		    while ((s = buf.readLine()) != null) 
		    {
			System.out.println("Error in extracting scripts : " + s);
		    }  		
		}
		catch(IOException e)
		{
		    System.out.println("IO Exception in readLine : " + e.getMessage());
		}
		buf.close();
	    }
	    else
		System.out.println("Successfully extracted " + scriptType);
	}
	catch(InterruptedException e)
	{
	    System.out.println("Interrupted Exception in proc.waitFor in  FindScripts : "  + e.getMessage());
	}
	catch(Exception e)
	{
	    System.out.println("Exception in method in FindScripts : "  + e.getMessage()); 
	}
    }

    public boolean foundScripts()
    {
	GUI.summaryReport[currentIndex] += "\n\n\nScript Analysis";
	GUI.summaryReport[currentIndex] += "\n---------------\n";
	
	GUI.detailedReport[currentIndex] += "\n\n\nScript Analysis";
	GUI.detailedReport[currentIndex] += "\n---------------\n";
	
	long prelen  = new File(preInstallFile).length();
	long postlen = new File(postInstallFile).length();
	
	// If any scripts are found, return true at the end of method
	if ((prelen > 0L) || (postlen > 0L))
	{
	    System.out.println("There are some scripts");
	}
	// No scripts found , so return false
	else 
	{
	    System.out.println("There are no scripts");
	    GUI.summaryReport[currentIndex] += "\nNo scripts found\n";
	    GUI.detailedReport[currentIndex] += "\nNo scripts found\n";
	    return false;
	}
	return true;
    }

    //  Method to find out if any script violates security policies
    public void anyScriptViolation()
    {
	String scriptPath  = Constants.DEMODIR  + "scripts/" + currentPackage + ".preinstall.sh";
	long filelen  = new File(scriptPath).length();
	//System.out.println("Pre Script length is : " + filelen);
	if (filelen > 0L)
	{
	    System.out.println("\nChecking for pre script violations");
	    scriptType = "Pre-Install";
	    isViolatingScript(scriptPath);
	}
	System.out.println("Finished checking pre-install scripts ");

	scriptPath  = Constants.DEMODIR  + "scripts/" + currentPackage + ".postinstall.sh";
	filelen  = new File(scriptPath).length();
	//System.out.println("Post Script length is : " + filelen);
	if (filelen > 0L) 
	{
	    System.out.println("\nChecking for post script violations");
	    scriptType = "Post-Install";
	    isViolatingScript(scriptPath); 
	}
	System.out.println("Finished checking post-install scripts ");
    }

    // Method to determine if the particular script violates a security policy
    private void isViolatingScript(String scriptPath)
    {
	try 
	{
	    String execStr = "/bin/sh " + Constants.DEMODIR + "tools/checkscripts.sh " + scriptPath;
	    System.out.println(execStr);
	    Process proc = theSystem.exec(execStr);
	    proc.waitFor();
	    int val = proc.exitValue();
	    // Normal Termination (We get the result on the output stream)
	    if (val == 0) 
	    { 
		BufferedReader buf = new BufferedReader(new InputStreamReader(proc.getInputStream()));
		try
		{
		    summaryViolations = "";
		    detailedViolations = "";
		    String s;		
		    while((s = buf.readLine()) != null)
		    {
			// Since the message received starts with "@@@@"
			s = s.substring(5).trim();
			System.out.println(scriptType+ " script violation : " + s);
			int index = s.indexOf(":");
			// There is a brief summary  which can be displayed
			if (index >= 0)
			{
			    String strSummary = s.substring(0 , index).trim();

			    // This "type" of violation was already caught before , so dont add it to summary but add the details to the detailed part
			    if (summaryViolations.indexOf(strSummary) >= 0) 
			    {
				// The violation has to be added to Detailed List irrespective of whether the type had been caught in summary before
				// Note that this is being done in case of duplicate messages for the same violation
				if (detailedViolations.indexOf(s) < 0) 
				{
				    detailedViolations = detailedViolations + "\n" + s;
				    GUI.scriptViolations.add(s);
				}
				continue;
			    }
			    // This violation is uncaught before and hence is to be caught and displayed
			    if (CommonMethods.displayViolation(strSummary))
			    {
				System.out.println("To be displayed : " +strSummary);
				summaryViolations = summaryViolations + "\n" + strSummary;
				detailedViolations = detailedViolations + "\n" + s;
				GUI.scriptViolations.add(s);
			    }
			}			
			// There is no brief summary for this violation 
			else
			{
			    // This violation is to be caught and displayed
			    if (CommonMethods.displayViolation(s))
			    {
				summaryViolations = summaryViolations + "\n" + s;
				detailedViolations = detailedViolations + "\n" + s;
				GUI.scriptViolations.add(s);
			    }			    
			}
		    }

		    // There is a script violation 
		    if (!summaryViolations.equals(""))
		    {
			GUI.summaryReport[currentIndex]  = GUI.summaryReport[currentIndex]  + "\n" + scriptType + " script  :  Violation(s) detected";
			GUI.summaryReport[currentIndex]  = GUI.summaryReport[currentIndex]  + "\n\t" + summaryViolations + "\n";
			GUI.detailedReport[currentIndex] = GUI.detailedReport[currentIndex] + "\n" + scriptType + " script  :  Violation(s) detected";
			GUI.detailedReport[currentIndex] = GUI.detailedReport[currentIndex] + "\n\t" + detailedViolations + "\n";
		    }
		    // There are no script violations detected
		    else
		    {
			GUI.summaryReport[currentIndex] = GUI.summaryReport[currentIndex] + "\n" + scriptType + " script  :  No violation detected\n";
			GUI.detailedReport[currentIndex] = GUI.detailedReport[currentIndex] + "\n" + scriptType + " script  :  No violation detected\n";
		    }
		}
		catch(IOException e)
		{
		    System.out.println("IO Exception in readLine in isViolatingScript : " + e.getMessage());
		}
		buf.close();
	    }
	}
	catch(InterruptedException e)
	{
	    System.out.println("Interrupted Exception in proc.waitFor in isViolatingScript : "  + e.getMessage());
	}
	catch(Exception e)
	{
	    System.out.println("Exception in method in  isViolatingScript : "  + e.getMessage()); 
	}
    }

//     // Method to determine if a particular violation is to be displayed or not. 
//     // If a violation is not be displayed it is supposed to not exist at all. 
//     private boolean toBeDisplayed(String violation)
//     {	
// 	violation = violation.toUpperCase();
// 	//System.out.println("Violation caught is : "  + violation  + "    Length is : " + violation.length());
// 	if ((new String("CREATE IN NOT OWNED DIRECTORIES").startsWith(violation)) && (GUI.jcbCreateNotOwn.isSelected())) return false;
// 	if ((new String("WRITE TO OWNED BUT MODIFIED FILES").startsWith(violation)) && (GUI.jcbWriteOwnMod.isSelected())) return false;
// 	if ((new String("WRITE TO FILES IN NOT OWNED DIRECTORIES").startsWith(violation)) && (GUI.jcbWriteNotOwn.isSelected())) return false;
// 	if ((new String("WRITE TO FILES SHARED BY OTHER PACKAGES").startsWith(violation)) && (GUI.jcbWriteShared.isSelected())) return false;
// 	if ((new String("ADD USERS").startsWith(violation)) && (GUI.jcbAddUsers.isSelected())) return false;
// 	if ((new String("UPDATE SYSTEM SERVICES").startsWith(violation)) && (GUI.jcbUpdateSysServices.isSelected())) return false;
// 	if ((new String("MODIFY SHARED LIBRARIES").startsWith(violation)) && (GUI.jcbModifyLibs.isSelected())) return false;
// 	if ((new String("PERFORM NETWORK OPERATIONS").startsWith(violation)) && (GUI.jcbPerformNetworkOps.isSelected())) return false;
// 	if ((new String("DELETE FILES").startsWith(violation)) && (GUI.jcbDeleteFiles.isSelected())) return false;
// 	if ((new String("EXECUTE UNKNOWN COMMANDS").startsWith(violation)) && (GUI.jcbExecuteUnCmd.isSelected())) return false;
// 	return true;
//     }

} // end class

