

/*  Copyright (C) 2002 Secure Systems Lab, Stony Brook. 

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program (See file COPYING in the main directory); 
    if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*   Written by Tapan Kamat tkamat@cs.sunysb.edu */

import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import java.io.File;

/**
 * Title: Uinter.java
 * Description: This is the User Interface for the RPMSHIELD tool
 * Copyright:    Copyright (c) 2002
 * Company: Secure & Reliable Systems Laboratory , Stony Brook , NY
 * @author : Tapan Kamat
 * @version 1.0
 */

public class GUI extends JFrame
{
    Container contentpane;

    JMenuBar jmbBar;

    JMenu jmFile;
    JMenuItem jmiOpen;
    JMenuItem jmiQuit;

    JMenu jmOperations;
    JMenuItem jmiQuery;
    JMenuItem jmiTestInstall;
    JMenuItem jmiVerifyDigitalSign;    

    JMenu jmPreferences;
    JMenuItem jmiGeneral;
    JMenuItem jmiRPMDirectories;
    JMenuItem jmiRPMSearchOptions;

    JMenu jmHelp;
    JMenuItem jmiContents;
    JMenuItem jmiAbout;

    JToolBar jtToolBar;
    JButton jtbQuery;
    JButton jtbTestInstall;
    JButton jtbVerifyDigitalSign;

    JPanel jpDirTree;
    JScrollPane jspDirTree;

    CheckTree cTree;
    DefaultTreeModel treeModel;
    FileNode mainRoot;
    TreeEventListener treeEventListener;

    JPanel jpButtonPanel;
    static JButton jbInstall;

    JPanel jpDetails;
    JTabbedPane jtpPane;
    JScrollPane jtpPackageInfoPanel;
    static JTextArea jtaPackageInfo;
    JScrollPane jtpFileListPanel;
    static JTextArea jtaFileList;
    JScrollPane jtpDependenciesPanel;
    static JTextArea jtaDependencies;
    JPanel jtpPolicyPanel;
    JPanel policyP = new JPanel();

    final JLabel jlSecurityPolicies;
    static JCheckBox jcbCreateNotOwn;
    static JCheckBox jcbWriteOwnMod;
    static JCheckBox jcbWriteNotOwn;
    static JCheckBox jcbWriteShared;
    static JCheckBox jcbAddUsers;
    static JCheckBox jcbUpdateSysServices;
    static JCheckBox jcbModifyLibs;
    static JCheckBox jcbPerformNetworkOps;
    static JCheckBox jcbDeleteFiles;
    static JCheckBox jcbExecuteUnCmd;
    static JCheckBox jcbAllowAllOps;

    JPanel jpActionProgress;
    final JLabel jlPackageLabel; 
    static JLabel jlCurrentPackage;
    static JLabel jlCurrentAction;
    static JProgressBar jpbProgressBar;

    static javax.swing.Timer timer;
    static int progressvalue;
    static int taskDone = 0;
    final static int ONE_SECOND = 1000;

    static boolean createNotOwn = false, writeOwnMod = false, writeNotOwn = false, writeShared = false;
    static boolean addUsers = false , updateSysServices = false  ,modifyLibs = false, performNetworkOps = false; 
    static boolean deleteFiles = false ,executeUnCmd = false , allowAllOps = false;

    GeneralOptionsDialog objGeneralOptionsDialog;
    static String currentDirectory = "";
    static Vector sharedDirectories = new Vector();

    RPMDirectoriesDialog objRPMDirectoriesDialog;
    static Vector rpmDirectories = new Vector();
    
    FindPackage objFP[];
    int installindex;

    static Vector scriptViolations = new Vector();
    
    static int currentIndex;
    static String summaryReport[];
    static String detailedReport[];
    
    int totalInstalls;
    static Vector preInstallList;
    static Vector installList;

    Runtime theSystem = Runtime.getRuntime();

    public GUI() 
    {
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
        contentpane = getContentPane();
        setLocation(0,0);
	System.out.println("VERSION  is : " + Constants.REDHATVERSION);
        setTitle("RPM Shield");
        setSize(Constants.UINTER_WIDTH,Constants.UINTER_HEIGHT);
	System.out.println("Width : " + Constants.UINTER_WIDTH);
	System.out.println("Height : " + Constants.UINTER_HEIGHT);

        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setResizable(false);
        addWindowListener(new WindowAdapter()
        {
            public void windowClosing(WindowEvent e)
            {

            }
        });

        /* Menu and Menu components starts */
        jmbBar = new JMenuBar();

        jmFile = new JMenu("File");
        jmiOpen = new JMenuItem("Open");
	jmiOpen.setVisible(false);
        jmiQuit = new JMenuItem("Quit");
        jmFile.add(jmiOpen);
        jmFile.add(jmiQuit);

        jmOperations= new JMenu("Operations");
        jmiQuery = new JMenuItem("Query");
        jmiTestInstall = new JMenuItem("Test Installation");
        jmiVerifyDigitalSign = new JMenuItem("Verify Digital Signature");
	jmiVerifyDigitalSign.setVisible(false);
        jmOperations.add(jmiQuery);
        jmOperations.add(jmiTestInstall);
        jmOperations.add(jmiVerifyDigitalSign);

        jmPreferences = new JMenu("Preferences");
        jmiGeneral = new JMenuItem("General");
        jmiRPMDirectories = new JMenuItem("RPM Directories");
        jmiRPMSearchOptions = new JMenuItem("RPM Search Options");
	jmiRPMSearchOptions.setVisible(false);
        jmPreferences.add(jmiGeneral);
        jmPreferences.add(jmiRPMDirectories);
        jmPreferences.add(jmiRPMSearchOptions);

        jmHelp = new JMenu("Help");
        jmiContents = new JMenuItem("Contents");
        jmiAbout = new JMenuItem("About");
        jmHelp.add(jmiContents);
        jmHelp.add(jmiAbout);

        jmbBar.add(jmFile);
        jmbBar.add(jmOperations);
        jmbBar.add(jmPreferences);
        jmbBar.add(jmHelp);

        setJMenuBar(jmbBar);
        /* Menu and Menu components ends*/

        /* Toolbar starts */
        jtToolBar = new JToolBar();
        jtToolBar.setMargin(new Insets(0,0,0,0));
        jtToolBar.setPreferredSize(new Dimension(Constants.UINTER_WIDTH,(int)Math.round(0.0359 * Constants.UINTER_HEIGHT)));
        jtToolBar.setFloatable(false);
        jtToolBar.setBorder(BorderFactory.createEtchedBorder());
        contentpane.add("North",jtToolBar);

        jtbQuery = new JButton("Query");
        jtbTestInstall = new JButton("Test Installation");
        jtbVerifyDigitalSign = new JButton("Verify Dig. Sign");
	jtbVerifyDigitalSign.setVisible(false);

        jtToolBar.add(jtbQuery);
        jtToolBar.add(jtbTestInstall);
        jtToolBar.add(jtbVerifyDigitalSign);

        contentpane.add("North",jtToolBar);
        /* Toolbar ends */

        /* Directory Tree panel starts */
        jpDirTree = new JPanel(true);
        jpDirTree.setLayout(new BorderLayout());
        jpDirTree.setPreferredSize(new Dimension(Constants.TREE_PANEL_WIDTH,Constants.TREE_PANEL_HEIGHT));
        jpDirTree.setBorder(BorderFactory.createEtchedBorder());

	mainRoot = new FileNode(new CheckableObject(new File("")));
        treeModel = new DefaultTreeModel(mainRoot);	
	cTree = new CheckTree(treeModel);
	treeEventListener = new TreeEventListener(cTree);
	cTree.addTreeSelectionListener(treeEventListener);
	cTree.addTreeExpansionListener(treeEventListener);
	cTree.addCheckListener(treeEventListener);
	cTree.setRootVisible(false);
	initialTreePopulate();

        jspDirTree = new JScrollPane(cTree);
        jpDirTree.add("Center",jspDirTree);

        jpButtonPanel = new JPanel();
        jbInstall = new JButton("Install");
        jpButtonPanel.add(jbInstall);
        jpDirTree.add("South",jpButtonPanel);

        contentpane.add("West",jpDirTree);
        /* Directory Tree panel ends */

        /* Details Panel starts */
        jpDetails = new JPanel(true);
        jpDetails.setLayout(new BorderLayout());
        jpDetails.setPreferredSize(new Dimension(Constants.DETAILS_PANEL_WIDTH,Constants.DETAILS_PANEL_HEIGHT));
        jpDetails.setBorder(BorderFactory.createEtchedBorder());

        jtpPane = new JTabbedPane();
	jtaPackageInfo = new JTextArea();
	jtaPackageInfo.setEditable(false);
        jtpPackageInfoPanel = new JScrollPane(jtaPackageInfo);

	jtaFileList = new JTextArea();
	jtaFileList.setEditable(false);
        jtpFileListPanel = new JScrollPane(jtaFileList);

	jtaDependencies = new JTextArea();
	jtaDependencies.setEditable(false);
        jtpDependenciesPanel = new JScrollPane(jtaDependencies);

        jtpPolicyPanel = new JPanel();
        jtpPolicyPanel.setLayout(null);
        policyP = new JPanel();
	policyP.setLayout(null);

        policyP.setSize(new Dimension((int)Math.round(0.3516 * Constants.UINTER_WIDTH),(int)Math.round(0.6160 * Constants.UINTER_HEIGHT)));
        policyP.setLocation((int)Math.round(0.0156 * Constants.UINTER_WIDTH),(int)Math.round(0.0205 * Constants.UINTER_HEIGHT)); 

	jlSecurityPolicies = new JLabel("SECURITY POLICIES   : ");
        jcbCreateNotOwn = new JCheckBox("Create in Not Owned directories");
        jcbWriteOwnMod = new JCheckBox("Write to Owned but modified files");
	jcbWriteNotOwn = new JCheckBox("Write to files in Not Owned Directories");	
        jcbWriteShared = new JCheckBox("Write to files shared by other packages");	
	jcbAddUsers = new JCheckBox("Add users");
	jcbUpdateSysServices = new JCheckBox("Update system services");
	jcbModifyLibs = new JCheckBox("Modify Shared libraries");
	jcbPerformNetworkOps = new JCheckBox("Perform Network operations");
	jcbDeleteFiles = new JCheckBox("Delete files");
	jcbExecuteUnCmd = new JCheckBox("Execute unknown commands");
	jcbAllowAllOps = new JCheckBox("Allow all operations");
		
	int gap = (int)Math.round(0.0103 * Constants.UINTER_HEIGHT);
	int height = gap;
	jlSecurityPolicies.setSize(jlSecurityPolicies.getPreferredSize());
	jlSecurityPolicies.setLocation((int)Math.round(0.0078 * Constants.UINTER_WIDTH),height);
	height += jlSecurityPolicies.getPreferredSize().getHeight() +  2 * gap; 

	jcbCreateNotOwn.setSize(jcbCreateNotOwn.getPreferredSize());
	jcbCreateNotOwn.setLocation((int)Math.round(0.0078 * Constants.UINTER_WIDTH),height);
	int cboxheight = (int)jcbCreateNotOwn.getPreferredSize().getHeight(); 
	height += cboxheight + gap ;
	
	jcbWriteOwnMod.setSize(jcbWriteOwnMod.getPreferredSize());
	jcbWriteOwnMod.setLocation((int)Math.round(0.0078 * Constants.UINTER_WIDTH),height);
	height += cboxheight + gap;

	jcbWriteNotOwn.setSize(jcbWriteNotOwn.getPreferredSize());
	jcbWriteNotOwn.setLocation((int)Math.round(0.0078 * Constants.UINTER_WIDTH),height);
	height += cboxheight + gap;

	jcbWriteShared.setSize(jcbWriteShared.getPreferredSize());
	jcbWriteShared.setLocation((int)Math.round(0.0078 * Constants.UINTER_WIDTH),height);
	height += cboxheight + gap;

	jcbAddUsers.setSize(jcbAddUsers.getPreferredSize());
	jcbAddUsers.setLocation((int)Math.round(0.0078 * Constants.UINTER_WIDTH),height);
	height += cboxheight + gap;

	jcbUpdateSysServices.setSize(jcbUpdateSysServices.getPreferredSize());
	jcbUpdateSysServices.setLocation((int)Math.round(0.0078 * Constants.UINTER_WIDTH),height);
	height += cboxheight + gap;

	jcbModifyLibs.setSize(jcbModifyLibs.getPreferredSize());
	jcbModifyLibs.setLocation((int)Math.round(0.0078 * Constants.UINTER_WIDTH),height);
	height += cboxheight + gap;

	jcbPerformNetworkOps.setSize(jcbPerformNetworkOps.getPreferredSize());
	jcbPerformNetworkOps.setLocation((int)Math.round(0.0078 * Constants.UINTER_WIDTH),height);
	height += cboxheight + gap;

	jcbDeleteFiles.setSize(jcbDeleteFiles.getPreferredSize());
	jcbDeleteFiles.setLocation((int)Math.round(0.0078 * Constants.UINTER_WIDTH),height);
	height += cboxheight + gap;

	jcbExecuteUnCmd.setSize(jcbExecuteUnCmd.getPreferredSize());
	jcbExecuteUnCmd.setLocation((int)Math.round(0.0078 * Constants.UINTER_WIDTH),height);
	height += cboxheight + gap;

	jcbAllowAllOps.setSize(jcbAllowAllOps.getPreferredSize());
	jcbAllowAllOps.setLocation((int)Math.round(0.0078 * Constants.UINTER_WIDTH),height);
	height += cboxheight + gap;

	policyP.add(jlSecurityPolicies);
	policyP.add(jcbCreateNotOwn);
	policyP.add(jcbWriteOwnMod);
	policyP.add(jcbWriteNotOwn);
	policyP.add(jcbWriteShared);
	policyP.add(jcbAddUsers);
	policyP.add(jcbUpdateSysServices);
	policyP.add(jcbModifyLibs);
	policyP.add(jcbPerformNetworkOps);
	policyP.add(jcbDeleteFiles);
	policyP.add(jcbExecuteUnCmd);
	policyP.add(jcbAllowAllOps);

	jtpPolicyPanel.add(policyP);
        jcbAllowAllOps.addItemListener(new java.awt.event.ItemListener()
        {
            public void itemStateChanged(ItemEvent e)
            {
                if (e.getStateChange() == ItemEvent.SELECTED)
		{
		    jcbCreateNotOwn.setSelected(true);
		    jcbWriteOwnMod.setSelected(true);
		    jcbWriteNotOwn.setSelected(true);
		    jcbWriteShared.setSelected(true);
		    jcbAddUsers.setSelected(true);
		    jcbUpdateSysServices.setSelected(true);
		    jcbModifyLibs.setSelected(true);
		    jcbPerformNetworkOps.setSelected(true);
		    jcbDeleteFiles.setSelected(true);  
		    jcbExecuteUnCmd.setSelected(true);
		}
            }
        });

        jtpPane.add(jtpPackageInfoPanel,"Package Info");
        jtpPane.add(jtpFileListPanel,"File List");
        jtpPane.add(jtpDependenciesPanel,"Dependencies");
        jtpPane.add(jtpPolicyPanel,"Policy");
        jpDetails.add("Center",jtpPane);
    
	/* Progress Panel */
	jpActionProgress = new JPanel();
	jpActionProgress.setPreferredSize(new Dimension(Constants.DETAILS_PANEL_WIDTH,(int)Math.round(0.1027 * Constants.UINTER_HEIGHT)));
	jpActionProgress.setLayout(null);	

	jlPackageLabel = new JLabel("Package : ");
        //jlPackageLabel.setSize(new Dimension((int)Math.round(0.0508 * Constants.UINTER_WIDTH),(int)Math.round(0.0205 * Constants.UINTER_HEIGHT)));
	jlPackageLabel.setSize(jlPackageLabel.getPreferredSize());
	jlPackageLabel.setLocation((int)Math.round(0.0078 * Constants.UINTER_WIDTH),(int)Math.round(0.0103 * Constants.UINTER_HEIGHT));

	jlCurrentPackage = new JLabel("None");
        jlCurrentPackage.setSize(new Dimension(Constants.DETAILS_PANEL_WIDTH - (int)Math.round(0.0703 * Constants.UINTER_WIDTH),(int)Math.round(0.0205 * Constants.UINTER_HEIGHT)));
	jlCurrentPackage.setLocation((int)Math.round(0.0664 * Constants.UINTER_WIDTH),(int)Math.round(0.0103 * Constants.UINTER_HEIGHT));

	jlCurrentAction = new JLabel("");
        jlCurrentAction.setSize(new Dimension(Constants.DETAILS_PANEL_WIDTH - (int)Math.round(0.0156 * Constants.UINTER_WIDTH) ,(int)Math.round(0.0205 * Constants.UINTER_HEIGHT)));
	jlCurrentAction.setLocation((int)Math.round(0.0078 * Constants.UINTER_WIDTH),(int)Math.round(0.0411 * Constants.UINTER_HEIGHT));

        jpActionProgress.add(jlPackageLabel);
        jpActionProgress.add(jlCurrentPackage);
        jpActionProgress.add(jlCurrentAction);

	progressvalue = 0;
        jpbProgressBar = new JProgressBar();
        jpbProgressBar.setBorder(BorderFactory.createEtchedBorder());
        jpbProgressBar.setSize(new Dimension(Constants.DETAILS_PANEL_WIDTH - (int)Math.round(0.0156 * Constants.UINTER_WIDTH) ,(int)Math.round(0.0205 * Constants.UINTER_HEIGHT)));
	jpbProgressBar.setMinimum(0);
        jpbProgressBar.setValue(progressvalue);
	
	jpbProgressBar.setLocation((int)Math.round(0.0078 * Constants.UINTER_WIDTH),(int)Math.round(0.0719 * Constants.UINTER_HEIGHT));
        jpActionProgress.add(jpbProgressBar);

        jpDetails.add("South",jpActionProgress);

	timer = new javax.swing.Timer(ONE_SECOND, new ActionListener()
	{
            public void actionPerformed(ActionEvent evt)
	    {
		if (progressvalue + 2 <= 100)
		    progressvalue += 2;
		else
		    progressvalue = 0;
		jpbProgressBar.setValue(progressvalue);		
            }
        });

        contentpane.add("East",jpDetails);
        /* Details Panel ends */

        /* Adding Listeners starts */
        jmiOpen.addActionListener(new ActionListener()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		System.out.println("Open pressed...");
	    }
        });

        jmiQuit.addActionListener(new ActionListener()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		System.out.println("Quit pressed...");
		System.exit(0);
	    }
        });

        jmiQuery.addActionListener(new ActionListener()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		System.out.println("Query pressed...");
		queryFile();
	    }
        });

        jmiTestInstall.addActionListener(new ActionListener()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		System.out.println("Test Install pressed...");
		testFile();
	    }
        });

        jmiVerifyDigitalSign.addActionListener(new ActionListener()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		System.out.println("Verify Digital Sign pressed...");
	    }
        });

        jmiGeneral.addActionListener(new ActionListener()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		System.out.println("General pressed...");
		if (objGeneralOptionsDialog == null)
		{
		    objGeneralOptionsDialog = new GeneralOptionsDialog(); 
		    objGeneralOptionsDialog.show();
		}
		else
		    objGeneralOptionsDialog.show();
	    }
        });

        jmiRPMDirectories.addActionListener(new ActionListener()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		System.out.println("RPM Directories pressed...");
		if (objRPMDirectoriesDialog == null)
		{
		    objRPMDirectoriesDialog = new RPMDirectoriesDialog(); 
		    objRPMDirectoriesDialog.show();
		}
		else
		    objRPMDirectoriesDialog.show();
	    }
        });

        jmiRPMSearchOptions.addActionListener(new ActionListener()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		System.out.println("RPM Search Options pressed...");
		new RPMSearchOptionsFrame();
	    }
        });

        jmiContents.addActionListener(new ActionListener()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		System.out.println("Contents pressed...");
	    }
        });

        jmiAbout.addActionListener(new ActionListener()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		  System.out.println("About pressed...");
	    }
        });

	jtbQuery.addActionListener(new ActionListener()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		System.out.println("Query Package pressed in toolbar...");
		queryFile();
	    }
	 });

	jtbTestInstall.addActionListener(new ActionListener()
	{
	    public void actionPerformed(ActionEvent e)
	    {
      		System.out.println("Test Package pressed in toolbar...");
		testFile();
	    }
        });

        jtbVerifyDigitalSign.addActionListener(new ActionListener()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		System.out.println("Verify Digital Signature pressed in toolbar...");
	    }
        });

	jbInstall.addActionListener(new ActionListener()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		System.out.println("Install button pressed...");
		preInstall();
	    }
	});
        /* Adding Listeners ends */
    }

    public static void startProgressBar()
    {
	if (!timer.isRunning())
	    timer.start();
	progressvalue = 0;
	taskDone = 0;
    }

    public static void stopProgressBar()
    {
	try
	{
	    if ( timer.isRunning())
		timer.stop();
	    Toolkit.getDefaultToolkit().beep();
	    jlCurrentPackage.setText("None");
	    jlCurrentAction.setText("");
	    taskDone = 1;
	    jbInstall.setEnabled(true);
	    jpbProgressBar.setValue(jpbProgressBar.getMaximum());
	    Thread.sleep(300);
	    jpbProgressBar.setValue(jpbProgressBar.getMinimum());
	    System.out.println("Stopping Progress Bar ");
	}
	catch(InterruptedException e)
	{
	    System.out.println("Interrupted exception in stopProgressBar() " + e.getMessage());
	}
    }

    public static void resumeProgressBar()
    {
	if (!timer.isRunning())
	    timer.start();
    }

    public static void pauseProgressBar()
    {
	if (timer.isRunning()) 
	    timer.stop();
    }

    // Method to move all the files in the Download directory and thereby make the directory empty so that it can be used in the installation 
    // process along with the list of rpm files supplied
    private boolean moveDownloads()
    {
       try
       {
	   String source = Constants.DEMODIR + "downloads";
	   File sourceFile = new File(source);

	   if (sourceFile.exists())
	   {
	       File [] sourceList = sourceFile.listFiles();
	       if (sourceList.length > 0)
	       {
		   String destination = Constants.DEMODIR + "rpmstore/";
		   File destFile = new File(destination);
		   if (!destFile.exists())
		   {
		       if (destFile.mkdir())
			   System.out.println("Created destination directory");
		       else
		       {
			   System.out.println("Unable to create destination directory");
			   return false;
		       }
		   }
		   else
		       System.out.println("Destination directory exists"); 
		   System.out.println("Moving files .....");
		   for(int i =0; i < sourceList.length; i++)
		       sourceList[i].renameTo(new File (destination + sourceList[i].getName()));
	       }
	       else
		   System.out.println("Source directory is empty. No files moved ");
	   }
	   else
	       System.out.println("Source directory does not exist. No files moved ");
       }
       catch(Exception e)
       {
	   System.out.println("Exception in method copyDownloads   : "  + e.getMessage()); 
	   return false;
       }
       return true;
    }


    // Method to create a default model of the tree directory structure with the root node 
    protected void initialTreePopulate() 
    {	
	TreePath mainRootPath = new TreePath(new Object[] {mainRoot});
	File [] roots = File.listRoots();	
	if (roots.length <= 0)	    return;
	Arrays.sort(roots);
	CommonMethods.sortForDirectories(roots);
	
	FileNode rootnode;
	TreePath rootpath;
	for (int i = 0; i < roots.length; i++)
	{	    
	    // get the real roots of this OS .eg  " / "
	    rootnode = new FileNode(new CheckableObject(roots[i]));
	    // get the root path for each OS root obtained 
	    rootpath = mainRootPath.pathByAddingChild(rootnode);		
	    cTree.fireTreeExpanded(rootpath);
	    mainRoot.add(rootnode);
	}
	mainRoot.setExplored(false);	
	cTree.expandPath(mainRootPath);
    }

    // Action taken when the query button is pressed 
    public void queryFile()
    {
	new Thread()
	{
	    public void run()
	    {
		String msg = "";
		Vector checkedList = treeEventListener.getCheckedList();
		if (checkedList.size() == 0)
		{
		    msg = "No files selected for querying .\nPlease select files for querying .";
		    JOptionPane.showMessageDialog(null,msg, "Query Package",JOptionPane.INFORMATION_MESSAGE);		
		    return;
		}

		jtbQuery.setEnabled(false);
		startProgressBar();		
		jlCurrentPackage.setText("All selected");
		jlCurrentAction.setText("Package querying in progress..........");

		String path = "";
		for (int i = 0; i < checkedList.size(); i++)
		    path = path + " " + (String) checkedList.elementAt(i); 
		
		Vector partialNames = CommonMethods.getPartialPackageName(path);
		Vector fullNames = CommonMethods.getFullPackageName(path); 
		
		String partialNamesList = "";
		for (int i = 0; i < partialNames.size(); i++)
		    partialNamesList = partialNamesList + " " + (String) partialNames.elementAt(i);
		
		Vector result = CommonMethods.queryRPMDatabase(partialNamesList);				
		for (int i = 0; i < result.size(); i++)
		{
		    String fullName = (String) fullNames.elementAt(i);
		    String output   = (String) result.elementAt(i);
		    System.out.println("Query package ->  Name + Version : " + fullName);
		    // New installation
		    if (output.endsWith("is not installed"))
			msg = msg + "\nThe package " + fullName + " is not installed on the system ";
		    // Already installed
		    else if (output.equals(fullName))
			msg = msg + "\nThe package " + fullName + " is already installed";
		    // Different version
		    else		    
			msg = msg + "\nThe package " + fullName + " is different from the installed version";
		}		
		pauseProgressBar();
		JOptionPane.showMessageDialog(null,msg, "Query Package",JOptionPane.INFORMATION_MESSAGE);
		stopProgressBar();
		jtbQuery.setEnabled(true);
	    }
	}.start();
    }

    // Action taken when the Test button is pressed
    public void testFile()
    {	
	new Thread()
	{
	    public void run()
	    {
		String msg = "";
		Vector checkedList = treeEventListener.getCheckedList();	
		if (checkedList.size() == 0)
		{
		    msg = "No files selected for Test Install .\nPlease select files for Test install .";
		    JOptionPane.showMessageDialog(null,msg, "Query Package",JOptionPane.INFORMATION_MESSAGE);		
		    return;
		}

		jtbTestInstall.setEnabled(false);		
		startProgressBar();	      
		jlCurrentPackage.setText("All selected");
		jlCurrentAction.setText("Test Installation in progress..........");

		String strFileList = "";
		for (int i = 0; i < checkedList.size(); i++)
		    strFileList = strFileList + " " + (String) checkedList.elementAt(i);
		
		try
		{
		    String execStr = "rpm -i --test " + strFileList;
		    Process proc = theSystem.exec(execStr);
		    proc.waitFor();
		    int val = proc.exitValue();		
		    
		    // After the process exec returns (finishes) along with an error message 
		    if (val > 0)
		    {
			BufferedReader buf = new BufferedReader(new InputStreamReader(proc.getErrorStream()));
			String s ;
			try
			{
			    while ((s = buf.readLine()) != null)	
			    {		
				// remove off the trailing or leading spaces , if any 
				s = s.trim();
				
				// first line contains the string "error" always in case of failed dependencies , so ignore it 
				if (s.startsWith("error:"))
				    continue;	
				
				// In case of failed installation, get the package name for which the test install failed
				int indexofneeded = s.indexOf("is needed by");
				if (indexofneeded >= 0)
				{		
				    String tempStr = s.substring(indexofneeded);
				    //System.out.println("Error string : " + tempStr);
				    
				    StringTokenizer strTok = new StringTokenizer(tempStr, " ");
				    int totalTokens = strTok.countTokens();
				    String strPkgName = "" ;
				    for (int i = 1; i <= totalTokens ; i++)
				    {
					strPkgName = strTok.nextToken();
					// we need only the 4th token to get our package name 
					if (i >= 4) break;
				    }
				    // If the package name is not yet added to the list , add it
				    if (msg.indexOf(strPkgName) < 0)
					msg = msg + "\n" + strPkgName + " test installation failed";
				}
				else
				{
				    //System.out.println("Possibly a package upgrade"); 
				    msg = "Test installation of selected packages was successful";		    
				    jtbTestInstall.setEnabled(true);
				    return;
				}
			    }
			}
			catch(IOException e)
			{
			    System.out.println("IO Exception in readline in testFile: " + e.getMessage());
			}
			buf.close();
		    }
		    // No error message given , so installation can go thru
		    else
		    {		   
			System.out.println("Val returned  <= 0"); 
			msg = "Test installation of selected packages was successful";
		    } 		
		}
		catch(InterruptedException e)
		{
		    System.out.println("Interrupted Exception in proc.waitFor in testfile : "  + e.getMessage());
		}
		catch(Exception e)
		{
		    System.out.println("Exception in method testfile : "  + e.getMessage()); 
		}
		finally
		{
		    pauseProgressBar();
		    JOptionPane.showMessageDialog(null,msg, "Test Installation",JOptionPane.INFORMATION_MESSAGE);
		    stopProgressBar();
		    jtbTestInstall.setEnabled(true);
		}	
	    } 
	}.start();
    } 


    // Pre-Installation actions taken after Install button is pressed
    public void preInstall()
    {	
	new Thread()
	{	
	    public void run()
	    {			
		jbInstall.setEnabled(false);			
		Vector checkedList = treeEventListener.getCheckedList();
		int numberofpackages = checkedList.size();
		
		System.out.println("List of " + numberofpackages + " selected packages for Install");
		if (numberofpackages == 0)
		{
		    JOptionPane.showMessageDialog(null,"You have not selected any packages for installation .\nPlease select packages to be installed .",
						  "No packages selected", JOptionPane.INFORMATION_MESSAGE);
		    stopProgressBar();
		    return;
		}
		
		/* Temporary commenting for testing purposes */

// 		if    (!jcbCreateNotOwn.isSelected() 
//                     && !jcbWriteOwnMod.isSelected() 
//                     && !jcbWriteNotOwn.isSelected() 
// 		    && !jcbAddUsers.isSelected()  
// 		    && !jcbUpdateSysServices.isSelected()  
// 		    && !jcbModifyLibs.isSelected()  
// 		    && !jcbPerformNetworkOps.isSelected()  
// 		    && !jcbDeleteFiles.isSelected()  
// 		    && !jcbExecuteUnCmd.isSelected()) 
// 		{
// 		    JOptionPane.showMessageDialog(null,"You have not specified a security policy",
// 						  "No security policy found", JOptionPane.INFORMATION_MESSAGE);
// 		    stopProgressBar();
// 		    return;
// 		}
		
		startProgressBar();
		
		// Move all the files from Download directory to the Store directory
		jlCurrentAction.setText("Creating a Downloads directory if necessary..........");
		if (!moveDownloads())
		{
		    pauseProgressBar();
		    JOptionPane.showMessageDialog(null,"File move from the downloads directory failed .",
						  "File Move Failure", JOptionPane.INFORMATION_MESSAGE);
		    stopProgressBar();
		    return;
		}
		
		jlCurrentAction.setText("Performing a Test Installation..........");
		String strFileList = "";
		for (int i = 0; i < numberofpackages; i++)
		{
		    strFileList = strFileList + " " + (String) checkedList.elementAt(i);
		}
		
		FetchDependencies objFetchDeps = new FetchDependencies(strFileList);
		objFetchDeps.testInstall();
		//if (!objFetchDeps.proceed) return;
		
		if (!objFetchDeps.isSuccess())
		{
		    pauseProgressBar(); 
		    JOptionPane.showMessageDialog(null,"Test installation failed. Please change your selection and try again.",
						  "Test Installation Failure", JOptionPane.INFORMATION_MESSAGE);
		    stopProgressBar();
		    return;
		}
		System.out.println("The Test Install was successful and the installation can proceed ");
		
		// count the number of installations to be done
		totalInstalls = 0;

		// This is a vector containing the list of the files which will ultimately be checked for pre-install violations
		preInstallList = new Vector();
		installList = new Vector();

		// Add all the newly downloaded files first to the pre-install list 		
		String strDownloadDir = Constants.DEMODIR + "downloads";
		File downloadDir = new File(strDownloadDir);
		if (downloadDir.exists())
		{
		    File [] downloadFiles = downloadDir.listFiles();			
		    if (downloadFiles != null)
		    {
			int totalDownloaded = downloadFiles.length;
			for (int i = 0 ; i < totalDownloaded ; i++)
			    preInstallList.add(downloadFiles[i].getAbsolutePath());
		    }
		}

		// Add all the checked files now to the pre-install list	
		for (int i = 0 ; i < numberofpackages ; i++)
		    preInstallList.add((String) checkedList.elementAt(i));

		String summaryReportHeader  = "\t\t Installation Report Summary\n\t\t ---------------------------\n";
		String detailedReportHeader = "\t\t Installation Report Details\n\t\t ---------------------------\n";

		totalInstalls = preInstallList.size();
		summaryReport  = new String[totalInstalls];
		detailedReport = new String[totalInstalls];

		currentIndex = 0;
		objFP = new FindPackage[totalInstalls];
		installindex = 0;

		for (int i = 0 ; i < totalInstalls; i++)
		{
		    jlCurrentPackage.setText((String)preInstallList.elementAt(i));
		    // Get all info about each package
		    install((String)preInstallList.elementAt(i));
		    currentIndex++;
		}

		jlCurrentPackage.setText("");
		jlCurrentAction.setText("Displaying Pre-Installation Report Summary..........");
		pauseProgressBar();

		PreInstallReport objPIR = new PreInstallReport(summaryReport,detailedReport);		
		if (objPIR.continueInstallation)
		{		    
		    resumeProgressBar();
		    System.out.println("All script violations caught as below : "); 
		    for (int i = 0 ; i < scriptViolations.size() ; i++)
			System.out.println(scriptViolations.elementAt(i));
		    jlCurrentAction.setText("Installing package..........");
		    System.out.println("Files being installed : ");		    
		    Upgrade objUp = new Upgrade(objPIR.installList);
		    objUp.sortInstallList();

		    // Install actions for every selected package
		    for (int i = 0; i < objUp.sortedInstallList.size(); i++)
		    {
			String absPath = (String) objUp.sortedInstallList.elementAt(i);
			System.out.println("Installing : " + absPath);
			jlCurrentPackage.setText(absPath);
			int installationType = 0;
			for (int j = 0; j < objFP.length; j++)
			{
			    // If this is the package being installed
			    if (absPath.equals(objFP[j].currentPackagePath))
			    {
				installationType = objFP[j].installationType;
				String packagename = CommonMethods.getPackageNameWithVersion(absPath);				

				// Write the violating create file names to the /tmp/create
				String createList = "";
				for (int p = 0 ; p < objFP[j].createNotOwnViolations.size(); p++)
				    createList += (String) objFP[j].createNotOwnViolations.elementAt(p) + "\n";				
				System.out.println("Create in not owned directory : \n" + createList);
				CommonMethods.addToCreateFile(packagename,createList);
				
				// Write the violating write file names to the /tmp/write
				String writeList = "";
				for (int p = 0 ; p < objFP[j].writeOwnModViolations.size(); p++)
				    writeList += (String) objFP[j].writeOwnModViolations.elementAt(p) + "\n";	
				//System.out.println("Write to owned but modified  : \n" + writeList);
				for (int p = 0 ; p < objFP[j].writeNotOwnViolations.size(); p++)
				    writeList += (String) objFP[j].writeNotOwnViolations.elementAt(p) + "\n";	
				//System.out.println("Write to not owned directory : \n" + writeList);
				for (int p = 0 ; p < objFP[j].writeSharedViolations.size(); p++)
				{
				    String s = (String) objFP[j].writeSharedViolations.elementAt(p);	
				    if (writeList.indexOf(s) < 0)
					writeList += s + "\n";
				}
				System.out.println("Write to files : \n" + writeList);				
				CommonMethods.addToWriteFile(packagename,writeList);
				break;
			    }
			}
			CommonMethods.createSymLink(absPath);
			showResult(objUp.doUpgrade(absPath,installationType));
		    }
		}
		else
		{
		    System.out.println("Aborting package");
		}
		stopProgressBar();
	    }
	}.start();
    }


    // Actual installation after pre-install finishes
    public void install(String strSelectedfile)
    {
	try
	{
	    File selectedfile = new File(strSelectedfile);
	    String absPath = selectedfile.getAbsolutePath();	    
	    System.out.println("Installation for the package : " + absPath);

	    String filename = selectedfile.getName();
	    String filepath = selectedfile.getParent();

	    summaryReport[currentIndex] = "\nFile name  :  " + filename;
	    summaryReport[currentIndex] = summaryReport[currentIndex] + "\n\nPath  :  " + filepath;
	    detailedReport[currentIndex] = "\nFile name  :  " + filename;
	    detailedReport[currentIndex] = detailedReport[currentIndex] + "\n\nPath  :  " + filepath;

	    jlCurrentAction.setText("Finding Installation Type..........");
	    objFP[installindex] = new FindPackage(absPath);
	    objFP[installindex].getInstallationType();

	    if (objFP[installindex].installationType == 1)
		return;	

	    // if the package is not already installed , we are interested in having the status information for the package
	    // so we increment the index by 1 , otherwise we dont , so that the previous index is overwritten 
	    // the variable index is being used from now on so that we still make sure that the index value is updated if there's 
	    // an error in any part of the code which follows . this ensures that everything later is at the proper index
	    int index = installindex++;

	    // This is the actual list of packages shortlisted by Pre-install
	    installList.add(absPath);

	    jlCurrentAction.setText("Getting Package Information..........");
	    objFP[index].getOwnedDirectories();
	    jlCurrentAction.setText("Checking against Security Policies..........");
	    objFP[index].categorizeFiles();
	    objFP[index].writeToReport();
	    
	    jlCurrentAction.setText("Searching for scripts..........");

	    FindScripts objFS = new FindScripts(absPath);
	    objFS.getScripts("preinstall");
	    objFS.getScripts("postinstall");

	    // Scripts were found, so script behavior is being checked	    
	    if (objFS.foundScripts())
	    {
		System.out.println("Checking for script violations");
		jlCurrentAction.setText("Analyzing scripts..........");
		CommonMethods.createSymLink(absPath);
		objFS.anyScriptViolation();
	    }
	}
	catch(Exception e) 
	{
	    System.out.println("Exception in method install  : ");
	    e.printStackTrace(); 
	}	
    }


    /*Overridden so we can exit when window is closed*/
    protected void processWindowEvent(WindowEvent e)
    {
	super.processWindowEvent(e);
	if (e.getID() == WindowEvent.WINDOW_CLOSING)
	{
	    System.exit(0);
	}
    }

    static public void showResult(String s)
    {
	pauseProgressBar();
	JOptionPane.showMessageDialog(null,s,"Installation Result",
				      JOptionPane.INFORMATION_MESSAGE);
	System.out.println("Show result");
    }

    static public int showRTM(String s) 
    {

	int n=0;
	
	pauseProgressBar();
	n = JOptionPane.showConfirmDialog(null,"The installation of this package will perform an operation\n" +
					  "not compatible with the security policy.\n" +
					  "The Runtime Monitor returned: \n" +
					  "\n" + s + "\n" +
					  "\nWould you still like the installation to continue ?\n",
					  "Runtime Monitor Output", JOptionPane.YES_NO_OPTION);

	if (n == JOptionPane.NO_OPTION) 
	{
	    stopProgressBar();
	    return 1;
	}
	else
	{
	    resumeProgressBar();
	    return 0;
	}
    }

    public static void main(String[] args) 
    {
	try 
	{
	    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
	}
	catch(Exception e) 
	{
	    System.out.println("Exception in settling Look&Feel in GUI" + e.getMessage());
	}
	/*   if (System.getProperty("user.name").equals("root")) { 
	     new proj();
	      }
	      else {
	      System.out.println("This program can be run only with root privileges");
	      }
	*/
	GUI objGUI = new GUI();
	objGUI.setVisible(true);
	
    }
}
