

/*  Copyright (C) 2002 Secure Systems Lab, Stony Brook. 

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program (See file COPYING in the main directory); 
    if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*   Written by Tapan Kamat tkamat@cs.sunysb.edu */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.util.*;

class PreInstallReport extends JDialog
{
    Runtime theSystem = Runtime.getRuntime();

    final JDialog reportDialog;
    Container dialogpane;

    JTabbedPane jtpReport;

    JPanel jpSummary;
    JScrollPane jspSummary;
    JPanel jpSummaryList;
    JTextArea jtaSummary[];
    JCheckBox jcbSummarySelection[];
    JButton jbSummaryViewScripts[];

    JPanel jpDetails;
    JScrollPane jspDetails;
    JPanel jpDetailsList;
    JTextArea jtaDetails[];
    JCheckBox jcbDetailsSelection[];
    JButton jbDetailsViewScripts[];

    BufferedReader buf;

    JPanel jpButtonPanel;
    JButton jbContinueInstall;
    JButton jbViewSummary;
    JButton jbViewDetails;
    JButton jbAbortInstall;

    MyActionListener myActionListener;

    File preInstall;
    File postInstall;

    boolean continueInstallation;
    int totalInstalls;
    Vector installList;
    Vector rejectList;
    String currentPackage;

    public PreInstallReport(String[] summaryReport,String[] detailedReport)
    {
	reportDialog = new JDialog();
	dialogpane = reportDialog.getContentPane();
	dialogpane.setLayout(null);
	reportDialog.setTitle("Pre-Installation Report");	    
	reportDialog.setModal(true);
	reportDialog.setResizable(false);	
	reportDialog.setLocation((int)Math.round(0.2188 * Constants.UINTER_WIDTH),(int)Math.round(0.1232 * Constants.UINTER_HEIGHT)); // 280,120
	reportDialog.setSize(new Dimension((int)Math.round(0.5469 * Constants.UINTER_WIDTH),(int)Math.round(0.7392 * Constants.UINTER_HEIGHT))); // 700,720	   
	reportDialog.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);       

	myActionListener = new MyActionListener();

        jtpReport = new JTabbedPane();
	jtpReport.setLocation((int)Math.round(0.0039 * Constants.UINTER_WIDTH),(int)Math.round(0.0103 * Constants.UINTER_HEIGHT)); // 5,10
	jtpReport.setSize(new Dimension((int)Math.round(0.5313 * Constants.UINTER_WIDTH),(int)Math.round(0.6160 * Constants.UINTER_HEIGHT))); // 680,600

	totalInstalls = summaryReport.length;
	installList = new Vector();
	rejectList = new Vector();

	fillSummaryPane(summaryReport);
	jtpReport.add(jpSummary,"Summary");

	fillDetailsPane(detailedReport);
	jtpReport.add(jpDetails,"Details");

	jpButtonPanel = new JPanel();
	jpButtonPanel.setLayout(null);
	jpButtonPanel.setLocation((int)Math.round(0.0039 * Constants.UINTER_WIDTH),(int)Math.round(0.6366 * Constants.UINTER_HEIGHT)); // 5,620
	jpButtonPanel.setSize(new Dimension((int)Math.round(0.5313 * Constants.UINTER_WIDTH),(int)Math.round(0.0719 * Constants.UINTER_HEIGHT)));//680,70	
							
	jbContinueInstall = new JButton("Continue"); //Continue Installation
	jbContinueInstall.setSize(new Dimension((int)Math.round(0.1328 * Constants.UINTER_WIDTH),(int)Math.round(0.0308 * Constants.UINTER_HEIGHT)));// 170,30
	jbContinueInstall.setLocation((int)Math.round(0.0156 * Constants.UINTER_WIDTH),(int)Math.round(0.0205 * Constants.UINTER_HEIGHT));//20,20
	jbContinueInstall.addActionListener(myActionListener);
	jpButtonPanel.add(jbContinueInstall);	

        jbViewSummary = new JButton("View Summary"); //View Summary
	jbViewSummary.setSize(new Dimension((int)Math.round(0.1328 * Constants.UINTER_WIDTH),(int)Math.round(0.0308 * Constants.UINTER_HEIGHT)));//150,30	
	jbViewSummary.setLocation((int)Math.round(0.2070 * Constants.UINTER_WIDTH),(int)Math.round(0.0205 * Constants.UINTER_HEIGHT));//265,20
	jbViewSummary.addActionListener(myActionListener);
	jpButtonPanel.add(jbViewSummary);
	jbViewSummary.setVisible(false);

	jbAbortInstall = new JButton("Abort"); //Abort Installation
	jbAbortInstall.setSize(new Dimension((int)Math.round(0.1172 * Constants.UINTER_WIDTH),(int)Math.round(0.0308 * Constants.UINTER_HEIGHT)));//150,30
	jbAbortInstall.setLocation((int)Math.round(0.3984 * Constants.UINTER_WIDTH),(int)Math.round(0.0205 * Constants.UINTER_HEIGHT));//510,20
	jbAbortInstall.addActionListener(myActionListener);
	jpButtonPanel.add(jbAbortInstall);

	jbViewDetails = new JButton("View Details");
	jbViewDetails.setSize(new Dimension((int)Math.round(0.1328 * Constants.UINTER_WIDTH),(int)Math.round(0.0308 * Constants.UINTER_HEIGHT)));//150,30
	jbViewDetails.setLocation((int)Math.round(0.2070 * Constants.UINTER_WIDTH),(int)Math.round(0.0205 * Constants.UINTER_HEIGHT));//265,20
	jbViewDetails.addActionListener(myActionListener);
	jpButtonPanel.add(jbViewDetails);

	dialogpane.add(jtpReport);
	dialogpane.add(jpButtonPanel);

	jtpReport.addChangeListener(new ChangeListener()
	{
	    public void stateChanged(ChangeEvent e)
	    {
		int i = jtpReport.getSelectedIndex();
		if (i == 0)
		{
		    GUI.jlCurrentAction.setText("Displaying Pre-Installation Report Summary..........");
		    jbViewSummary.setVisible(false);
		    jbViewDetails.setVisible(true);
		}
		else
		{
		    GUI.jlCurrentAction.setText("Displaying Pre-Installation Report Details..........");
		    jbViewDetails.setVisible(false);
		    jbViewSummary.setVisible(true);
		}
	    }
	});	
	continueInstallation = false;
	reportDialog.show();
    }
    
    private void fillSummaryPane(String[] summaryReport)
    {
	jpSummary = new JPanel();
	jpSummary.setLayout(null);

	jpSummaryList = new JPanel();
	jpSummaryList.setLayout(null);
	jpSummaryList.setLocation((int)Math.round(0.0078 * Constants.UINTER_WIDTH),(int)Math.round(0.0103 * Constants.UINTER_HEIGHT));//10,10

	jtaSummary = new JTextArea[totalInstalls];	
	jcbSummarySelection = new JCheckBox[totalInstalls];	
	jbSummaryViewScripts = new JButton[totalInstalls];

	int xcoord = (int)Math.round(0.4688 * Constants.UINTER_WIDTH);//600
	int ycoord = (int)Math.round(0.0103 * Constants.UINTER_HEIGHT);//10
	for (int i =0  ; i < totalInstalls; i++)
	{
	    final int j = i;
	    int tempXCoord = (int)Math.round(0.0078 * Constants.UINTER_WIDTH);//10
	    jtaSummary[i] = new JTextArea(summaryReport[i]);
	    Dimension textSize = jtaSummary[i].getPreferredSize();
	    int textWidth = (int) textSize.getWidth();
int textHeight = (int) textSize.getHeight();
jtaSummary[i].setLocation((int)Math.round(0.0078 * Constants.UINTER_WIDTH),ycoord);  //10
	    jtaSummary[i].setEditable(false);
 	    jpSummaryList.add(jtaSummary[i]);

	    ycoord = ycoord + textHeight + (int)Math.round(0.0078 * Constants.UINTER_HEIGHT);//10; 

	    String checkText = (String) GUI.preInstallList.elementAt(i);	   
	    if (GUI.installList.indexOf(checkText) >= 0)
	    {
		jcbSummarySelection[i] = new JCheckBox(checkText);
		Dimension checkSize = jcbSummarySelection[i].getPreferredSize();
		int checkWidth = (int) checkSize.getWidth();
		int checkHeight = (int) checkSize.getHeight();	    
		jcbSummarySelection[i].setSelected(true);
jcbSummarySelection[i].setLocation((int)Math.round(0.0078 * Constants.UINTER_WIDTH),ycoord);//10
		jcbSummarySelection[i].setSize(checkSize);
		installList.add(checkText);
		jcbSummarySelection[i].addItemListener(new java.awt.event.ItemListener()
		{
		    public void itemStateChanged(ItemEvent e)
		    {
			String text = ((JCheckBox)e.getSource()).getText();
			if (e.getStateChange() == ItemEvent.SELECTED)
			{			
			    if (installList.indexOf(text) < 0)
			    {
				installList.add(text);
				jcbDetailsSelection[j].setSelected(true);
				System.out.println("Added to accept list : " + text);
			    }
			    int entryIndex = rejectList.indexOf(text);
			    if (entryIndex >= 0)
			    {
				rejectList.remove(entryIndex);
				System.out.println("Removed from reject list : " + text);
			    }
			}
			if (e.getStateChange() == ItemEvent.DESELECTED)
			{			    
			    int entryIndex = installList.indexOf(text);
			    if (entryIndex >= 0)
			    {
				installList.remove(entryIndex);
				jcbDetailsSelection[j].setSelected(false);
				System.out.println("Removed from accept list : " + text);
			    }
			    if (rejectList.indexOf(text) < 0)
			    {
				rejectList.add(text);
				System.out.println("Added to reject list : " + text);
			    }
			}
		    }
		 });	    
		jpSummaryList.add(jcbSummarySelection[i]);
		
		tempXCoord = tempXCoord + checkWidth;

		currentPackage = CommonMethods.getPackageNameWithVersion(checkText);
		preInstall = new File(Constants.DEMODIR + "scripts/" + currentPackage + ".preinstall.sh");	
		postInstall = new File(Constants.DEMODIR +"scripts/" + currentPackage + ".postinstall.sh");	
		if ((preInstall.length() > 0) || (postInstall.length() > 0)) 
		{
		    jbSummaryViewScripts[i] = new JButton("View Scripts");
jbSummaryViewScripts[i].setLocation(tempXCoord + (int)Math.round(0.0156 * Constants.UINTER_WIDTH),ycoord); //20
		    jbSummaryViewScripts[i].setSize(jbSummaryViewScripts[i].getPreferredSize());
		    jbSummaryViewScripts[i].addActionListener(new ActionListener()
		    {
			public void actionPerformed(ActionEvent e)
			{
			    scriptViewer(reportDialog,j);
			}
		    });
		    jpSummaryList.add(jbSummaryViewScripts[i]);
tempXCoord = tempXCoord + (int)Math.round(0.0156 * Constants.UINTER_WIDTH) + (int)jbSummaryViewScripts[i].getPreferredSize().getWidth();//20
		}	    
		ycoord = ycoord + checkHeight;
	    }

	    if ((textWidth > (int)Math.round(0.4688 * Constants.UINTER_WIDTH)) || (tempXCoord > (int)Math.round(0.4688 * Constants.UINTER_WIDTH))) // 600,600
	    {
		if ((textWidth > tempXCoord) && (textWidth > xcoord))
		    xcoord = textWidth;
		else if ((tempXCoord > textWidth) && (tempXCoord > xcoord))
		    xcoord = tempXCoord;
	    }
	    ycoord = ycoord + (int)Math.round(0.0411 * Constants.UINTER_HEIGHT);//40;	    
	}
	// End of first for loop

	for (int i = 0 ; i < totalInstalls; i++)
	    jtaSummary[i].setSize(new Dimension(xcoord + (int)Math.round(0.0078 * Constants.UINTER_WIDTH),(int)jtaSummary[i].getPreferredSize().getHeight()));	//10
	
	jpSummaryList.setPreferredSize(new Dimension(xcoord + (int)Math.round(0.0234 * Constants.UINTER_WIDTH),ycoord + (int)Math.round(0.0205 * Constants.UINTER_HEIGHT)));  //30,20
	jspSummary = new JScrollPane(jpSummaryList);
	jspSummary.getVerticalScrollBar().setUnitIncrement((int)Math.round(0.0103 * Constants.UINTER_HEIGHT));//10
	jspSummary.setLocation((int)Math.round(0.0078 * Constants.UINTER_WIDTH),(int)Math.round(0.0103 * Constants.UINTER_HEIGHT));//10,10
	jspSummary.setSize(new Dimension((int)Math.round(0.5117 * Constants.UINTER_WIDTH),(int)Math.round(0.5749 * Constants.UINTER_HEIGHT)));//655,560
	jpSummary.add(jspSummary);
    }

    private void fillDetailsPane(String[] detailedReport)
    {
	jpDetails = new JPanel();
	jpDetails.setLayout(null);

	jpDetailsList = new JPanel();
	jpDetailsList.setLayout(null);
	jpDetailsList.setLocation((int)Math.round(0.0078 * Constants.UINTER_WIDTH),(int)Math.round(0.0103 * Constants.UINTER_HEIGHT));//10,10

	jtaDetails = new JTextArea[totalInstalls];	
	jcbDetailsSelection = new JCheckBox[totalInstalls];	
	jbDetailsViewScripts = new JButton[totalInstalls];

	int xcoord = (int)Math.round(0.4688 * Constants.UINTER_WIDTH);//600
	int ycoord = (int)Math.round(0.0103 * Constants.UINTER_HEIGHT);//10
	for (int i =0  ; i < totalInstalls; i++)
	{
	    final int j = i;
	    int tempXCoord = (int)Math.round(0.0078 * Constants.UINTER_WIDTH);//10
	    jtaDetails[i] = new JTextArea(detailedReport[i]);
	    Dimension textSize = jtaDetails[i].getPreferredSize();
	    int textWidth = (int) textSize.getWidth();
	    int textHeight = (int) textSize.getHeight();
	    jtaDetails[i].setLocation((int)Math.round(0.0078 * Constants.UINTER_WIDTH),ycoord); //10
	    jtaDetails[i].setEditable(false);
 	    jpDetailsList.add(jtaDetails[i]);

	    ycoord = ycoord + textHeight + (int)Math.round(0.0103 * Constants.UINTER_HEIGHT);//10 

	    String checkText = (String) GUI.preInstallList.elementAt(i);	   
	    if (GUI.installList.indexOf(checkText) >= 0)
	    {
		jcbDetailsSelection[i] = new JCheckBox(checkText);
		Dimension checkSize = jcbDetailsSelection[i].getPreferredSize();
		int checkWidth = (int) checkSize.getWidth();
		int checkHeight = (int) checkSize.getHeight();	    
		jcbDetailsSelection[i].setSelected(true);
jcbDetailsSelection[i].setLocation((int)Math.round(0.0078 * Constants.UINTER_WIDTH),ycoord);  //10
		jcbDetailsSelection[i].setSize(checkSize);

		// Nothing added to installList here as it was already added in  fillSummaryPane   method
		jcbDetailsSelection[i].addItemListener(new java.awt.event.ItemListener()
		{
		    public void itemStateChanged(ItemEvent e)
		    {
			String text = ((JCheckBox)e.getSource()).getText();
			if (e.getStateChange() == ItemEvent.SELECTED)
			{			
			    if (!jcbSummarySelection[j].isSelected())
				jcbSummarySelection[j].setSelected(true);
			}
			if (e.getStateChange() == ItemEvent.DESELECTED)
			{
			    if (jcbSummarySelection[j].isSelected())
				jcbSummarySelection[j].setSelected(false);
			}
		    }
		});	    
		jpDetailsList.add(jcbDetailsSelection[i]);
		
		tempXCoord = tempXCoord + checkWidth;

		currentPackage = CommonMethods.getPackageNameWithVersion(checkText);
		preInstall = new File(Constants.DEMODIR + "scripts/" + currentPackage + ".preinstall.sh");	
		postInstall = new File(Constants.DEMODIR + "scripts/" + currentPackage + ".postinstall.sh");	
		if ((preInstall.length() > 0) || (postInstall.length() > 0)) 
		{
		    jbDetailsViewScripts[i] = new JButton("View Scripts");
jbDetailsViewScripts[i].setLocation(tempXCoord + (int)Math.round(0.0156 * Constants.UINTER_WIDTH),ycoord);  //20
		    jbDetailsViewScripts[i].setSize(jbDetailsViewScripts[i].getPreferredSize());
		    jbDetailsViewScripts[i].addActionListener(new ActionListener()
		    {
			public void actionPerformed(ActionEvent e)
			{
			    scriptViewer(reportDialog,j);
			}
		    });
		    jpDetailsList.add(jbDetailsViewScripts[i]);
tempXCoord = tempXCoord + (int)Math.round(0.0156 * Constants.UINTER_WIDTH) + (int)jbDetailsViewScripts[i].getPreferredSize().getWidth();//20
		}	    
		ycoord = ycoord + checkHeight;
	    }

	    if ((textWidth > (int)Math.round(0.4688 * Constants.UINTER_WIDTH)) || (tempXCoord > (int)Math.round(0.4688 * Constants.UINTER_WIDTH)))//600,600
	    {
		if ((textWidth > tempXCoord) && (textWidth > xcoord))
		    xcoord = textWidth;
		else if ((tempXCoord > textWidth) && (tempXCoord > xcoord))
		    xcoord = tempXCoord;
	    }
	    ycoord = ycoord + (int)Math.round(0.0411 * Constants.UINTER_HEIGHT);//40
	}
	// End of first for loop

	for (int i = 0 ; i < totalInstalls; i++)
	    jtaDetails[i].setSize(new Dimension(xcoord + (int)Math.round(0.0078 * Constants.UINTER_WIDTH),(int)jtaDetails[i].getPreferredSize().getHeight()));	//10
	
	jpDetailsList.setPreferredSize(new Dimension(xcoord + (int)Math.round(0.0234 * Constants.UINTER_WIDTH),ycoord + (int)Math.round(0.0205 * Constants.UINTER_HEIGHT)));//30,20
	jspDetails = new JScrollPane(jpDetailsList);
	jspDetails.getVerticalScrollBar().setUnitIncrement((int)Math.round(0.0103 * Constants.UINTER_HEIGHT));//10
	jspDetails.setLocation((int)Math.round(0.0078 * Constants.UINTER_WIDTH),(int)Math.round(0.0103 * Constants.UINTER_HEIGHT));//10,10 
	jspDetails.setSize(new Dimension((int)Math.round(0.5117 * Constants.UINTER_WIDTH),(int)Math.round(0.5749 * Constants.UINTER_HEIGHT)));//655,560
	jpDetails.add(jspDetails);
    }

    // Listens to the push buttons. 
    private class MyActionListener implements ActionListener 
    {
        public void actionPerformed(ActionEvent e) 
	{
	    if (e.getSource() == jbContinueInstall)
	    {
		GUI.pauseProgressBar();
		GUI.jlCurrentPackage.setText("All selected");
		GUI.jlCurrentAction.setText("Test installation in  progress..........");
		boolean success = CommonMethods.testInstall(installList,rejectList);
		if (success)
		{
		    continueInstallation = true;		
		    reportDialog.dispose();
		}
		else
		{
		    GUI.jlCurrentPackage.setText("");
		    int i = jtpReport.getSelectedIndex();
		    if (i == 0)
			GUI.jlCurrentAction.setText("Displaying Pre-Installation Report Summary..........");
		    else
			GUI.jlCurrentAction.setText("Displaying Pre-Installation Report Details..........");
		}
	    }
	    else if (e.getSource() == jbViewSummary)
	    {
		jtpReport.setSelectedIndex(0);
	    }
	    else if (e.getSource() == jbAbortInstall)
	    {
		GUI.jlCurrentAction.setText("Aborting Installation..........");		
		GUI.stopProgressBar();
		reportDialog.dispose();
	    }
	    else if (e.getSource() == jbViewDetails)
	    {
		jtpReport.setSelectedIndex(1);
	    }
	}
    }

    private void scriptViewer(final JDialog reportDialog, int packageIndex)
    {
	final JDialog scriptDialog = new JDialog();
	Container contentPane = scriptDialog.getContentPane();
	contentPane.setLayout(null);	
	scriptDialog.setTitle("View Scripts"); 
	scriptDialog.setModal(true);
	scriptDialog.setResizable(false);
        scriptDialog.setLocation((int)Math.round(0.2343 * Constants.UINTER_WIDTH),(int)Math.round(0.2259 * Constants.UINTER_HEIGHT));//300,220
	scriptDialog.setSize((int)Math.round(0.5156 * Constants.UINTER_WIDTH),(int)Math.round(0.5339 * Constants.UINTER_HEIGHT));//660,520
	scriptDialog.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);

	final JTabbedPane jtpScriptPane = new JTabbedPane();
	jtpScriptPane.setSize(new Dimension((int)Math.round(0.4922 * Constants.UINTER_WIDTH),(int)Math.round(0.4415 * Constants.UINTER_HEIGHT)));//630,430
	jtpScriptPane.setLocation((int)Math.round(0.0078 * Constants.UINTER_WIDTH),(int)Math.round(0.0103 * Constants.UINTER_HEIGHT));//10,10

	JButton jbClose = new JButton("Close");
	jbClose.setSize(new Dimension((int)Math.round(0.0859 * Constants.UINTER_WIDTH),(int)Math.round(0.0308 * Constants.UINTER_HEIGHT)));//110,30
	jbClose.setLocation((int)Math.round(0.1445 * Constants.UINTER_WIDTH),(int)Math.round(0.4620 * Constants.UINTER_HEIGHT));//185,450
	jbClose.addActionListener(new ActionListener()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		scriptDialog.dispose();
		GUI.jlCurrentAction.setText("Displaying Pre-Installation Report Details..........");
	        reportDialog.show();
	    }
	});

	JButton jbViewModel = new JButton("View Model");
	jbViewModel.setSize(new Dimension((int)Math.round(0.1094 * Constants.UINTER_WIDTH),(int)Math.round(0.0308 * Constants.UINTER_HEIGHT)));//140,30
	jbViewModel.setLocation((int)Math.round(0.2852 * Constants.UINTER_WIDTH),(int)Math.round(0.4620 * Constants.UINTER_HEIGHT));//365,450
	jbViewModel.addActionListener(new ActionListener()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		new Thread()
		{	
		    public void run()
		    {
			modelViewer(reportDialog,jtpScriptPane.getTitleAt(jtpScriptPane.getSelectedIndex()));
		    }
		}.start();
	    }
	});

	currentPackage = CommonMethods.getPackageNameWithVersion(jcbSummarySelection[packageIndex].getText());

	preInstall = new File(Constants.DEMODIR + "scripts/" + currentPackage + ".preinstall.sh");	
	long filelen = preInstall.length();
	if (filelen > 0L)
	{
	    try
	    {
		BufferedReader buf = new BufferedReader(new FileReader(preInstall));
		String preInstallScript = "";
		String s;
		while((s = buf.readLine()) != null)
		    preInstallScript = preInstallScript + s + "\n";
		JTextArea jtaPreInstallScript = new JTextArea(preInstallScript);
		jtaPreInstallScript.setEditable(false);
		JScrollPane jspPreInstallScript = new JScrollPane(jtaPreInstallScript);
		jtpScriptPane.add(jspPreInstallScript,"Pre-Install Script");
	    }
	    catch(IOException e)
	    {
		System.out.println("IO Exception reading Pre-Install script" + e.getMessage());
	    }
	    catch(Exception e)
	    {
		System.out.println("Exception reading Pre-Install script" + e.getMessage());
	    }
	}

	postInstall = new File(Constants.DEMODIR + "scripts/" + currentPackage + ".postinstall.sh");	
	filelen = postInstall.length();
	if (filelen > 0L)
	{
	    try
	    {
		BufferedReader buf = new BufferedReader(new FileReader(postInstall));
		String postInstallScript = "";
		String s;
		while((s = buf.readLine()) != null)
		    postInstallScript = postInstallScript + s + "\n";
		JTextArea jtaPostInstallScript = new JTextArea(postInstallScript);
		jtaPostInstallScript.setEditable(false);
		JScrollPane jspPostInstallScript = new JScrollPane(jtaPostInstallScript);
		jtpScriptPane.add(jspPostInstallScript,"Post-Install Script");
	    }
	    catch(IOException e)
	    {
		System.out.println("IO Exception reading Post-Install script" + e.getMessage());
	    }
	    catch(Exception e)
	    {
		System.out.println("Exception reading Post-Install script" + e.getMessage());
	    }
	}
	contentPane.add(jtpScriptPane);  
	contentPane.add(jbClose);
	contentPane.add(jbViewModel);
	scriptDialog.show();
    }    

    private void modelViewer(final JDialog reportDialog,String tabTitle)
    {
	String scriptType = tabTitle;
 	if (tabTitle.indexOf("Pre-Install") >= 0)
 	    scriptType = "Pre-Install";
 	else
 	    scriptType = "Post-Install";

	try 
	{
	    String modelViewPath = "";
	    if (scriptType.equals("Pre-Install"))
	    {
		System.out.println("Opening pre - install model");
		modelViewPath  = Constants.DEMODIR  + "scripts/" + currentPackage + ".preinstall.ps";
	    }
	    else
	    {
		System.out.println("Opening post - install model");
		modelViewPath  = Constants.DEMODIR  + "scripts/" + currentPackage + ".postinstall.ps";
	    }

	    if (new File(modelViewPath).exists())
	    {
		String execStr = "gv " + modelViewPath;
		System.out.println(execStr);
		Process proc = theSystem.exec(execStr);
		proc.waitFor();
		int val = proc.exitValue();
		// Abnormal Termination (We get the result on the error stream)
		if (val != 0) 
		{ 
		    String error = "Error in viewing the " + scriptType + " script model";
		    JOptionPane.showMessageDialog(reportDialog,error,"Model viewer error",
						  JOptionPane.WARNING_MESSAGE);
		}		
	    }
	    else
	    {
		String error = "This " + scriptType + " script model does not exist";
		JOptionPane.showMessageDialog(null,error,"Model error",
					      JOptionPane.WARNING_MESSAGE);
	    }
	}
	catch(InterruptedException e)
	{
	    System.out.println("Interrupted Exception in proc.waitFor in modelViewer : "  + e.getMessage());
	}
	catch(Exception e)
	{
	    String errmsg = e.getMessage();
	    if ((errmsg.indexOf("gv: not found") >= 0)) 
	    {
		System.out.println("gv not installed");
		String error = "gv is not installed.\nVersion 3.5.8 or above expected for viewing script models.";
		JOptionPane.showMessageDialog(null,error,"Script Viewer Failure", JOptionPane.INFORMATION_MESSAGE);
	    }
	    else
	    {
		System.out.println("Exception in method in modelViewer : "  + e.getMessage()); 
		String error = "Error in viewing the " + scriptType + " script model";
		JOptionPane.showMessageDialog(null,error,"Model viewer error",
					      JOptionPane.WARNING_MESSAGE);
	    }
	}
    }

    private void printInstallList()
    {
	System.out.println("New installation list : ");
	for(int i = 0 ; i < installList.size(); i++)
	    System.out.println((String) installList.elementAt(i));
    }

    public static void main(String [] args)
    {
	String summary[] = {"Summary Report1","Summary Report2","Summary Report3","Summary Report4","Summary Report5"};
	String details[] = {"Detailed Report1","Detailed Report2","Detailed Report3","Detailed Report4","Detailed Report5"};
	PreInstallReport objTD = new PreInstallReport(summary,details);
	objTD.show();
    }

} // end of class

