

/*  Copyright (C) 2002 Secure Systems Lab, Stony Brook. 

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program (See file COPYING in the main directory); 
    if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*   Written by Tapan Kamat tkamat@cs.sunysb.edu */

import java.io.*;
import javax.swing.*;
import javax.swing.JTable;
import javax.swing.table.*;
import javax.swing.JScrollPane;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import javax.swing.JOptionPane;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

class RPMDirectoriesDialog extends JDialog 
{
    Container dialogpane;

    JTable table;
    MyTableModel tableModel;
    JScrollPane scrollPane;

    JPanel jpEastButtonPanel;
    JButton jbAdd;
    JButton jbRemove;

    JPanel jpSouthButtonPanel;
    JLabel jlCurrentDirectory;
    JTextField jtfCurrentDirectory;
    JFileChooser jfChooseFile;
    JButton jbBrowse;
    JButton jbOk;
    JButton jbApply;
    JButton jbCancel;

    String currentDirectory = System.getProperty("user.home");
    MyActionListener myActionListener;

    Vector vecRPMDirs;

    public RPMDirectoriesDialog()
    {
	super();
	dialogpane = getContentPane();
	dialogpane.setLayout(null);
	setModal(true);
	setResizable(false);
	setTitle("RPM Directories");	    
	setLocation(325,300);
	setSize(600,350);	   
	setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
	
        addWindowListener(new WindowAdapter()
        {
            public void windowClosing(WindowEvent e)
            {
		System.out.println("Window close clicked in RPM Directories");
		//cancelChanges();
            }
        });

	myActionListener = new MyActionListener();
	vecRPMDirs = new Vector();
	backupVector();

        tableModel = new MyTableModel();
	table = new JTable(tableModel);
	table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	table.setPreferredScrollableViewportSize(new Dimension(450,190));
        scrollPane = new JScrollPane(table);
	scrollPane.setLocation(10,20);
	scrollPane.setSize(new Dimension(460,200));

	// East Panel
	jpEastButtonPanel = new JPanel();
	jpEastButtonPanel.setLayout(null);
	jpEastButtonPanel.setLocation(480,20);
	jpEastButtonPanel.setSize(new Dimension(100,200));
	
	jbAdd = new JButton("Add");
	jbAdd.setLocation(10,70);
	jbAdd.setSize(new Dimension(90,30));	    
	jbAdd.addActionListener(myActionListener);

	jbRemove = new JButton("Remove");
	jbRemove.setLocation(10,120);
	jbRemove.setSize(new Dimension(90,30));	    
	jbRemove.addActionListener (myActionListener);

	jpEastButtonPanel.add(jbAdd);
	jpEastButtonPanel.add(jbRemove);

	// South Panel
	jpSouthButtonPanel = new JPanel();
	jpSouthButtonPanel.setLayout(null);
	jpSouthButtonPanel.setLocation(10,220);
	jpSouthButtonPanel.setSize(new Dimension(600,120));

	jlCurrentDirectory = new JLabel("Current Directory : ");
	jlCurrentDirectory.setLocation(10,20);	
	jlCurrentDirectory.setSize(new Dimension(175,20));
	jtfCurrentDirectory = new JTextField(300);
	jtfCurrentDirectory.setLocation(140,20);
	jtfCurrentDirectory.setSize(new Dimension(320,20));	
	jtfCurrentDirectory.setText(System.getProperty("user.home"));

	jfChooseFile = new JFileChooser("user.home");
	jfChooseFile.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
	jbBrowse = new JButton("Browse");
	jbBrowse.setLocation(480,15);
	jbBrowse.setSize(new Dimension(90,30));	    
	jbBrowse.addActionListener(myActionListener);
	
	jbOk = new JButton("OK");
	jbOk.setLocation(160,60);
	jbOk.setSize(new Dimension(80,30));	    
	jbOk.addActionListener(myActionListener);

	jbApply = new JButton("Apply");
	jbApply.setLocation(260,60);
	jbApply.setSize(new Dimension(80,30));	    
	jbApply.addActionListener(myActionListener);
	jbApply.setEnabled(false);

	jbCancel = new JButton("Cancel");
	jbCancel.setLocation(360,60);
	jbCancel.setSize(new Dimension(80,30));	    
	jbCancel.addActionListener(myActionListener);

	jpSouthButtonPanel.add(jlCurrentDirectory);
	jpSouthButtonPanel.add(jtfCurrentDirectory);
	jpSouthButtonPanel.add(jbBrowse);	
	jpSouthButtonPanel.add(jbOk);
	jpSouthButtonPanel.add(jbApply);
	jpSouthButtonPanel.add(jbCancel);

        //Add the scroll pane and button panels to the contentpane.
        dialogpane.add(scrollPane);
	dialogpane.add(jpEastButtonPanel);
	dialogpane.add(jpSouthButtonPanel);	
    }
    
    /* Listens to the push as well as radio buttons. */
    private class MyActionListener implements ActionListener 
    {
        public void actionPerformed(ActionEvent e) 
	{
	    if (e.getSource() == jbAdd)
	    {
		String rowcontent = jtfCurrentDirectory.getText().trim();
		if (!rowcontent.equals(""))
		    tableModel.addRow(rowcontent);
	    }
	    else if (e.getSource() == jbRemove)
	    {
		int rowindex = table.getSelectedRow();		
		if (rowindex >= 0)
		    tableModel.removeRow(rowindex);
	    }
	    if (e.getSource() == jbBrowse)
	    {
		int fileChoice = jfChooseFile.showOpenDialog(dialogpane);
		if (fileChoice == JFileChooser.APPROVE_OPTION)
		{
		    File dirName = jfChooseFile.getSelectedFile();
		    String dirAbsPath = dirName.getAbsolutePath();
		    jtfCurrentDirectory.setText(dirAbsPath);
		}
	    }
	    else if (e.getSource() == jbOk)
	    {
		applyChanges();
		dispose();
	    }
	    else if (e.getSource() == jbApply)
	    {
		applyChanges();
	    }
	    else if (e.getSource() == jbCancel)
	    {
		cancelChanges();
	    }
	}
    }

    private void applyChanges()
    {
	backupVector();
	saveToIni();
	jbApply.setEnabled(false);
    }

    private void cancelChanges()
    {
	if (! jbApply.isEnabled())
	    return;
	
	int size = GUI.rpmDirectories.size();
	for (int i = size - 1; i >= 0 ; i--)
	    tableModel.removeRow(i);
	
	for (int i = 0; i < vecRPMDirs.size(); i++)
	{
	    String s = (String) vecRPMDirs.elementAt(i);
	    tableModel.addRow(s);
	}	
	jbApply.setEnabled(false);
    }

    private void displayWarning()
    {
	String msg = "Please use the Apply / Cancel button to save or cancel the changes";
	JOptionPane.showMessageDialog(null,msg,"Unsaved Changes",JOptionPane.INFORMATION_MESSAGE);		    
    }

    private void backupVector()
    {    
	Enumeration enumRPMDirs = GUI.rpmDirectories.elements();
	vecRPMDirs.removeAllElements();
	while(enumRPMDirs.hasMoreElements())
	    vecRPMDirs.add((String) enumRPMDirs.nextElement());
    }

    private void saveToIni()
    {
	BufferedWriter buf = null;
	try
	{
	    String ininame = Constants.DEMODIR + "ini/rpmdirs.ini";
	    buf = new BufferedWriter(new FileWriter(ininame));
	    for (int i = 0 ; i < vecRPMDirs.size(); i++)
	    {
		String s = (String) vecRPMDirs.elementAt(i);
		buf.write(s + "\n");
	    }		
	    buf.close();
	}
	catch(IOException e)
	{
	    System.out.println("IO Exception in readLine : " + e.getMessage());
	}
	catch (Exception e)
	{
	    System.out.println("Exception in constructor of MyTableModel : " + e.getMessage());
	}
    }

    class MyTableModel extends AbstractTableModel 
    {
        final String[] columnNames = {"RPM Directory Paths"};
	Vector data = new Vector();
	
	public MyTableModel()
	{
	    BufferedReader buf = null;
	    try
	    {
		String ininame = Constants.DEMODIR + "ini/rpmdirs.ini";
		buf = new BufferedReader(new FileReader(ininame));
		String s;
		while ((s = buf.readLine()) != null)
		{
		    s = s.trim();
		    if (!s.equals(""))
		    {
			data.addElement(s);
			GUI.rpmDirectories.add(s);
			vecRPMDirs.add(s);
		    }
		}		
		buf.close();
	    }
	    catch(IOException e)
	    {
		System.out.println("IO Exception in readLine : " + e.getMessage());
	    }
	    catch (Exception e)
	    {
		System.out.println("Exception in constructor of MyTableModel : " + e.getMessage());
	    }
	}

	public int getColumnCount() {
	    return columnNames.length;
	}
        
        public int getRowCount() {
            return data.size();
        }

        public String getColumnName(int col) {
            return columnNames[0];
        }

	public Object getValueAt(int row, int col) {
	    return data.elementAt(row);
	}

	public void addRow(String rowname)
	{
	    if (! data.contains(rowname))
	    {
		data.addElement(rowname);
		fireTableDataChanged();
		GUI.rpmDirectories.add(rowname);
		jbApply.setEnabled(true);
	    }
	    else
		System.out.println("Already contained : " + rowname);	
	}
	public void removeRow(int rowindex)
	{
	    data.remove(rowindex);
	    fireTableDataChanged();
	    GUI.rpmDirectories.remove(rowindex);
	    jbApply.setEnabled(true);	    
	}
    }
    
    public static void main(String[] args) 
    {
        new RPMDirectoriesDialog().show();
    }
}
