

/*  Copyright (C) 2002 Secure Systems Lab, Stony Brook. 

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program (See file COPYING in the main directory); 
    if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*   Written by Tapan Kamat tkamat@cs.sunysb.edu */

import javax.swing.*;
import javax.swing.table.*;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.JScrollPane;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import javax.swing.JOptionPane;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.text.*;

class RPMSearchOptionsFrame extends JFrame {
    private boolean DEBUG = true;

    JPanel jpEastButtonPanel;
    JButton jbAdd;
    JButton jbRemove;

    JPanel jpSouthButtonPanel;
    JButton jbOk;
    JButton jbApply;
    JButton jbCancel;

    JTable table;
    MyTableModel myModel;
    DefaultCellEditor myCellEditor;
    JTextField url;
    
    Vector data;

    
    public RPMSearchOptionsFrame() {
        super("RPM Search Options");

	Container contentpane = getContentPane();
	setLocation(250,200);
	setSize(new Dimension(600,350));

        myModel = new MyTableModel();
        table = new JTable( myModel);
	url = new JTextField();

	//myCellEditor = new DefaultCellEditor(url);
	//table.setCellEditor(myCellEditor);

        table.setPreferredScrollableViewportSize(new Dimension(350,190));
	table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        //Create the scroll pane and add the table to it. 
        JScrollPane scrollPane = new JScrollPane(table);
	scrollPane.setLocation(20,10);
	scrollPane.setSize(new Dimension(360,200));

	jpEastButtonPanel = new JPanel();
	jpEastButtonPanel.setLocation(440,150);
	jpEastButtonPanel.setPreferredSize(new Dimension(100,150));
	
	jbAdd = new JButton("Add");
	jbRemove = new JButton("Remove");

	jpEastButtonPanel.add(jbAdd);
	jpEastButtonPanel.add(jbRemove);

	jpSouthButtonPanel = new JPanel();
	jpSouthButtonPanel.setLocation(10,280);
	jpSouthButtonPanel.setPreferredSize(new Dimension(400,50));
	
	jbOk = new JButton("Ok");
	jbApply = new JButton("Apply");
	jbCancel = new JButton("Cancel");
	
	jpSouthButtonPanel.add(jbOk);
	jpSouthButtonPanel.add(jbApply);
	jpSouthButtonPanel.add(jbCancel);

        //Add the scroll pane to this window.
        contentpane.add("West",scrollPane);
	contentpane.add("East",jpEastButtonPanel);
	contentpane.add("South",jpSouthButtonPanel);

        addWindowListener(new WindowAdapter() 
	{
            public void windowClosing(WindowEvent e) 
	    {
		//System.exit(0);
		dispose();
            }
        });

        jbAdd.addActionListener(new ActionListener()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		System.out.println("Add ...");								
		jbApply.setEnabled(true);
		myModel.addRow();
	    }
        });

        jbRemove.addActionListener(new ActionListener()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		System.out.println("Remove...");
		int selectedrow = table.getSelectedRow();
		System.out.println("Selected row is : "  + selectedrow);
		myModel.removeRow(selectedrow);
	    }
        });


        jbOk.addActionListener(new ActionListener()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		System.out.println("OK...");
		//System.exit(0);
		dispose();
	    }
        });


        jbApply.addActionListener(new ActionListener()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		if (table.isEditing())
		{
		    String newdata = ((JTextComponent)table.getEditorComponent()).getText();
		    System.out.println("The new data is : " + newdata );
		    data.remove(data.size() -1 );		
		    myModel.setValueAt(newdata,data.size(),0);
		    //myCellEditor.cancelCellEditing();
		    jbApply.setEnabled(false);
		}
	    }
        });


        jbCancel.addActionListener(new ActionListener()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		System.out.println("Cancel...");
	    }
        });
	
	pack();
	setVisible(true);
    }

    class MyTableModel extends AbstractTableModel 
    {
        final String columnName;

	public MyTableModel()
	{
	    data = new Vector();
	    columnName = "RPM Search URLs"; 
	    data.addElement("ftp://ftp.rpmfind.net");
	    data.addElement("ftp://ftp.rufus.org");
	    data.addElement("ftp://ftp.abcd.net");
	    data.addElement("ftp://ftp.myrpm.org");
	    data.addElement("ftp://ftp.rpmdp.org");
	}

	/* Added by Tapan */
	public void addRow()
	{
	    myModel.setValueAt("",data.size(),0);
	}

	public void removeRow(int row) 
	{
	    if (row <= data.size()) 
	    {
		data.removeElementAt(row);
	    }
	    myModel.fireTableDataChanged();
	    printDebugData();
	}	

        public int getColumnCount() {
            return 1;
        }
        
        public int getRowCount() {
            return data.size();
        }

        public String getColumnName(int col) {
            return columnName;
        }

        public Object getValueAt(int row, int col) {
            return data.elementAt(row);
        }
        /*
         * Don't need to implement this method unless your table's
         * editable.
         */
        public boolean isCellEditable(int row, int col) {
            //Note that the data/cell address is constant,
            //no matter where the cell appears onscreen.
            return false;
        }

        /*
         * Don't need to implement this method unless your table's
         * data can change.
         */
        public void setValueAt(Object value, int row, int col) 
	{
	    data.insertElementAt(value,row);
	    //fireTableCellUpdated(row, col);
	    fireTableDataChanged();

	    System.out.println("Printing data:");
	    printDebugData();
        }

        private void printDebugData() {
            int numRows = getRowCount();

            for (int i=0; i < numRows; i++) 
	    {
                System.out.print("    Row " + i + ":");
		System.out.print("  " + data.elementAt(i));
		System.out.println();
            }
            System.out.println("--------------------------"); 
        }
    }

    public static void main(String[] args) {
        RPMSearchOptionsFrame frame = new RPMSearchOptionsFrame();
        frame.pack();
        frame.setVisible(true);
    }
}
