

/*  Copyright (C) 2002 Secure Systems Lab, Stony Brook. 

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program (See file COPYING in the main directory); 
    if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*   Written by Tapan Kamat tkamat@cs.sunysb.edu */

import javax.swing.UIManager;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import java.io.*;
import javax.swing.event.*;
import java.util.*;

public class TreeEventListener implements TreeSelectionListener , TreeExpansionListener , CheckListener
{
    CheckTree cTree ; 
    Vector checkedList;
    TreeAction treeAction;
    
    public TreeEventListener(CheckTree checkTree)
    {
	cTree = checkTree;
	checkedList = new Vector();
	treeAction = new TreeAction();
    }

    public void treeCollapsed(TreeExpansionEvent e) 
    {
	//System.out.println("Tree-collapsed event detected");
    }

    public void treeExpanded(TreeExpansionEvent event)
    {
	//System.out.println("Tree-expanded event detected");
	
	// dont use JTree's getSelectedPath here as you cannot get a proper selected path in case the user clicks on a node handle (+/- sign)
	TreePath path = event.getPath();		
	String strPath = CommonMethods.getPath(path.toString());		
	if (strPath.equals("")) strPath = "/";

	//System.out.println("Str Path : " + strPath);

	File parentfile = new File(strPath);		
	if ((parentfile == null) || (parentfile.getName() == ""))
	{
	    System.out.println("No file object created for: "+ strPath);
	    return;
	}
	else 
	    // If the node which is being expanded is a file , just return from this method immediately
	    if (!parentfile.isDirectory())        return;
	
	FileNode parent = (FileNode)path.getLastPathComponent();	
	// if the dircectory has been already explored (populated with only a dummy)
	if (parent.isExplored()) 
	{
	    parent.removeAllChildren();             //remove the dummy node attached to the node to be expanded
	    parent.setExplored(false);
	}

	// if the directory has not been already populated
	if (parent.getChildCount() == 0)
	{ 
	    //try to get a file array of this file node's children
	    File [] children = parentfile.listFiles();
	    // if there are no children in this directory then return
	    if (children.length <= 0) 
	    {
		((DefaultTreeModel)cTree.getModel()).nodeStructureChanged(parent);
		return;
	    }
	    Arrays.sort(children);
	    CommonMethods.sortForDirectories(children);
	    parent.expandNode(children);
	    ((DefaultTreeModel)cTree.getModel()).nodeStructureChanged(parent);
	}
    }

    // This method caters to a Tree Selection event 
    public void valueChanged(TreeSelectionEvent event)
    {
	//System.out.println("Tree-selection event fired");		
	TreePath selectedPath = cTree.getSelectionPath();
	String absPath = CommonMethods.getPath(selectedPath.toString());		
	//System.out.println("The tree path is : "  + absPath);
	treeAction.populateDetails(absPath);
	treeAction.populateFileList(absPath);
	treeAction.populateDependencies(absPath);
    }

    // This method caters to a Tree Node Check event 
    public void valueChanged(CheckEvent event) 
    {
	TreePath path = cTree.getSelectionPath();	
	if (path == null) return;
	
	// This gets the entire Treepath of selected file name in a string
	String strPath = CommonMethods.getPath(path.toString());				    
	if (strPath.equals("")) strPath = "/";
	File parentfile = new File(strPath);		
	
	if(parentfile == null) 
	{
	    System.out.println("Could not create file object for: "+ strPath);
	    return;
	}
	else 
	{
	    // if the checked / unchecked  node is a directory
	    if (parentfile.isDirectory())
	    {			
		//System.out.println("This is a directory");
		// Get the reference to the File node of the parent thru the last path component 
		FileNode parent = (FileNode)path.getLastPathComponent();
		cTree.fireTreeExpanded(path);
		//System.out.println("This node is a directory with " + parent.getChildCount() + " children");
		
		for (int i = 0; i < parent.getChildCount(); i++) 
		{
		    // Get the reference to each child node so that we can check / uncheck it 
		    FileNode child = (FileNode) parent.getChildAt(i);
		    TreePath childpath = path.pathByAddingChild(child);
		    
		    // Get the path from root to this node in string format
		    String strChildPath = CommonMethods.getPath(childpath.toString());				    
		    
		    // Create a file object for this node as now we wish to find out if this node is a file
		    File childfile = new File(strChildPath);		
		    
		    // if the child of this checked / unchecked node is a file then check / uncheck it as required
		    if (childfile.isFile()) 
		    {
			// if the particular parent node is checked 
			if (event.isChecked())
			{
			    child.setChecked(true);
			    // If the package name is not present in the vector , add it to the vector
			    if (checkedList.indexOf(strChildPath) < 0)
				checkedList.addElement(strChildPath);
			}
			// the particular parent node is unchecked    
			else 
			{
			    child.setChecked(false);
			    // If the package name is present in the vector , remove it from the vector
			    if (checkedList.indexOf(strChildPath) >= 0)
			    {
				int entryIndex = checkedList.indexOf(strChildPath);
				checkedList.remove(entryIndex);
			    }
			} 				    
		    }
		} // end of  for  loop
	    }
	    // if the checked / unchecked node itself is a file
	    else
	    {
		//System.out.println("This is a file");
		// if the particular parent node is checked 
		if (event.isChecked())
		{
		    // If the package name is not present in the vector , add it
		    if (checkedList.indexOf(strPath) < 0)
			checkedList.addElement(strPath);
		}
		// the particular parent node is unchecked    
		else
		{
		    // Get the reference to the File node of the parent thru the last path component 
		    FileNode parent = (FileNode)path.getLastPathComponent();			    
		    ((FileNode)parent.getParent()).setChecked(false);
		    
		    // If the package name is present in the vector , remove it from the vector
		    if (checkedList.indexOf(strPath) >= 0)
		    {
			int entryIndex = checkedList.indexOf(strPath);
			checkedList.remove(entryIndex);
		    }				
		}
	    } // end of the   if directory condition
	}// end of   if parentfile is null condition 
	
	/*
	System.out.println("List of selected packages");
	for(int i = 0 ; i < checkedList.size() ; i++)
	    System.out.println((String)checkedList.elementAt(i));
	*/
	
	System.out.println("No of selected packages : " + checkedList.size());
    }   // end of method 

    // This method just returns the list of the checked Files in a Vector
    public Vector getCheckedList()
    {
	return checkedList;
    }
}


