

/*  Copyright (C) 2002 Secure Systems Lab, Stony Brook. 

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program (See file COPYING in the main directory); 
    if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*   Written by Tapan Kamat tkamat@cs.sunysb.edu */

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.swing.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

public class Upgrade 
{
    Runtime theSystem = Runtime.getRuntime();    
    Vector origInstallList = new Vector();
    String origInstallPaths = "";
    HashMap requiresList = new HashMap();
    Vector providesList = new Vector();
    Vector sortedInstallList = new Vector();

    public Upgrade(Vector origInstallList) 
    {
	this.origInstallList = origInstallList;
	for (int i = 0 ; i < this.origInstallList.size(); i++)
	    origInstallPaths += " "  + (String) origInstallList.elementAt(i);
    }

    public void sortInstallList()
    {
	getRequiredList();
	getProvidedList();
	getProviders();
 	System.out.println("Sorted List : ");
 	for (int i = 0 ; i < sortedInstallList.size(); i++)
 	    System.out.println((String) sortedInstallList.elementAt(i));	    

    }

    private void getRequiredList()
    {
	try
	{
	    for (int i = 0 ; i < origInstallList.size();i++)
	    {	
		String path = (String) origInstallList.elementAt(i);
		String execStr = "rpm -U --test " + path;
		Process proc = theSystem.exec(execStr);
		proc.waitFor();
		int val = proc.exitValue();		
		
		if (val > 0)
		{
		    BufferedReader buf = new BufferedReader(new InputStreamReader(proc.getErrorStream()));
		    String s ;
		    try
		    {
			Vector requiredPkg = new Vector();
			while ((s = buf.readLine()) != null)	
			{		
			    // remove off the trailing or leading spaces , if any 
			    s = s.trim();
			    
			    // first line contains the string "error" always in case of failed dependencies , so ignore it 
			    if (s.startsWith("error:"))
				continue;	
			    
			    // In case of failed installation, get the package name for which the test install failed
			    int indexofneeded = s.indexOf("is needed by");
			    if (indexofneeded >= 0)
			    {					    
				String tempStr = s.substring(0,indexofneeded - 1);
				int indexsep = tempStr.indexOf(" ");
				// Get only the dependency name here 
				if (indexsep > 0)
				    tempStr = tempStr.substring(0, indexsep);
				if (!requiredPkg.contains(tempStr))
				    requiredPkg.add(tempStr);
			    }
			}
			if (!requiredPkg.isEmpty())
			    requiresList.put(path,requiredPkg);
		    }
		    catch(IOException e)
		    {
			System.out.println("IO Exception in readline in getRequiredList : " + e.getMessage());
		    }
		    buf.close();
		}
	    }
	    System.out.println("Printing Required Packages");
	    Iterator e = requiresList.entrySet().iterator();
	    while(e.hasNext())
		System.out.println(e.next());

	}
	catch(InterruptedException e)
	{
	    System.out.println("Interrupted Exception in proc.waitFor in getRequiredList : "  + e.getMessage());
	}
	catch(Exception e)
	{
	    System.out.println("Exception in method getRequiredList : "  + e.getMessage()); 
	}
    }
	    
    private void getProvidedList()
    {
	try
	{
	    String paths = origInstallPaths;
	    String execStrArray[] = {"rpm", "-qp", "--queryformat", "*****\n[%{PROVIDES}\n]", paths};	
	    Process proc = theSystem.exec(execStrArray);
	    proc.waitFor();
	    int val = proc.exitValue();
	    BufferedReader buf = new BufferedReader(new InputStreamReader(proc.getInputStream()));
	    String s = "";
	    try
	    {
		String key = "";
		int keyindex = 0;
		boolean keyfound = false;		
		Vector value = new Vector();
		String strValue = "";
		while((s = buf.readLine())!= null)
		{
		    // If this string does not contain ***** , its a provided packagename for the package denoted by key
		    if (s.indexOf("*****") < 0)
		    {
			if (! value.contains(s))
			{
			    value.add(s.trim());
			    strValue += "," + s.trim(); 
			}
		    }
		    // Ths string contains ***** , so its a packagename. this is a new packagename(key) found
		    else
		    {
			// A key has been found earlier and we want to demarcate the line between the values of the previous key and this new one
			if (keyfound)
			{
			    // put key,value pair in HashMap after removing the first comma from  strValue and inserting a comma at the end . 
			    // This has significance in pattern matching later
			    providesList.add(key + "::" + strValue.substring(1) + ",");
			    value.removeAllElements();
			    strValue = "";
			}
			// This is the first time we are getting any key
			else
			    keyfound = true;
			key = (String) origInstallList.elementAt(keyindex++);
		    }
		}
		// This is where we put the final packagename along with the capabilities it provides (last pair)
		providesList.add(key + "::" + strValue.substring(1) + ",");
	    }
	    catch(IOException e)
	    {
		System.out.println("IO Exception in readline in getProvidedList: " + e.getMessage());
	    }
	    buf.close();
	    System.out.println("\nPrinting Provided Packages\n");
	    for (int i = 0 ; i < providesList.size(); i++)
		System.out.println(providesList.elementAt(i));
	}
	catch(InterruptedException e)
	{
	    System.out.println("Interrupted Exception in proc.waitFor in getProvidedList: "  + e.getMessage());
	}
	catch(Exception e)
	{
	    System.out.println("Exception in method getProvidedList: "  + e.getMessage()); 
	}		
    }

    private void getProviders()
    {
	for (int i = 0 ; i < origInstallList.size(); i++)
	{
	    String path = (String) origInstallList.elementAt(i);
	    if (! requiresList.containsKey(path))
	    {
		sortedInstallList.add(path);
		System.out.println("Sorted : " + path);
	    }
	}

 	Iterator iterRequires = requiresList.keySet().iterator();	

	// Now we need to add every element which has a dependency problem after its dependencies have been satisfied
 	while(iterRequires.hasNext())
	{
	    String key = (String) iterRequires.next();
	    Vector value = (Vector) requiresList.get(key);
	    int providersize = providesList.size();		
	    int requiressize = value.size();
	    // Getting every requirement value from Vector for each failed package installation 
	    for (int i = 0 ; i < requiressize; i++)
	    {
		String requires = (String) value.elementAt(i) + ",";
		// For every requirement , check in the provides list values to see if there is a match 
		// This can be optimized to check only for those keys whose deps. have not yet been satisfied
		// In that case,the other packages already have been added to the sorted list and hence they wont be a problem
		for (int j = 0 ; j < providersize; j++)
		{
		    String provider = (String) providesList.elementAt(j);
		    // Found a match here , so now we need to place the package providing this capability in the correct order
		    if (provider.indexOf(requires) >= 0)
		    {
			provider = provider.substring(0,provider.indexOf("::"));
			System.out.println("Found : " + requires  + "\tin\t" + provider);			
			orderPackages(key,provider); 
		    }
		}
	    }
	    // If key has not already been added before then add it now to the end of the sorted list
	    if (! sortedInstallList.contains(key))
		sortedInstallList.add(key);
	}	
    }

    private void orderPackages(String key,String provider)
    {
	// The key has been added previously , so we need to insert the provider at a location just before the key 
	if (sortedInstallList.contains(key))
	{
	    int keyindex = sortedInstallList.indexOf(key);	    
	    // Provider has not yet been added,so insert it in the place above the key 
	    if (! sortedInstallList.contains(provider))
	    {
		sortedInstallList.insertElementAt(provider,keyindex);
	    }
	    // Provider has already been added,but if its above the key, dont do anything
	    // else remove it from its current location and put it above the key
	    else
	    {
		int providerindex = sortedInstallList.indexOf(provider);
		if (providerindex > keyindex)
		{
		    sortedInstallList.remove(providerindex);
		    sortedInstallList.insertElementAt(provider,keyindex);
		}		
	    }
	}
	// The key has not yet been added, so just add the provider to the end of the sorted list if its not added as yet
	else
	{
	    if (! sortedInstallList.contains(provider))
		sortedInstallList.add(provider);
	}	    
    }       

    public String doUpgrade(String path,int installationType) 
    {	
	String upgradeResult = "";
	DatagramSocket dgSocket = null;
	DatagramPacket dgPacket = null;
	Process proc;
	try 
	{
	    int size = 1024;
	    byte[] buffer = new byte[size];
	    dgSocket = new DatagramSocket(1500);
	    dgPacket = new DatagramPacket(buffer, size);
	    try 
	    {
		int flag = 0;
		final String tracerPath = Constants.DEMODIR + "tools/tracer -f " + Constants.DEMODIR + "tools/mapfile.runtime";
		String execStr = tracerPath + " rpm -U " + path;
		proc = theSystem.exec(execStr);
		while (flag == 0) 
		{
		    System.out.println("Waiting for data");
		    dgSocket.receive(dgPacket);
		    System.out.println("Received data");
		    byte[] data = dgPacket.getData();

		    int port = dgPacket.getPort();
		    InetAddress address = dgPacket.getAddress();
		    int length = dgPacket.getLength();
		    String msg = new String(data,0,length - 1);

		    System.out.println("Message recvd : " + msg);
		    System.out.println("Length of converted msg : " + msg.length());

		    // if the message received starts with "@@@@"
		    if (msg.startsWith("@@@@")) 
		    {
			System.out.println("Entered if part");
			msg = msg.substring(5).trim();
			
			// This violation type has not been encountered before in the script analysis
			if (! GUI.scriptViolations.contains(msg))
			{			    
			    String str = msg;
			    int index = str.indexOf(":");
			    // We are interested in suppressing messages which belong to the class of violations which are acceptable
			    if (index >= 0)
				str = str.substring(0, index).trim();
			    
			    // This violation is to be displayed
			    if (CommonMethods.displayViolation(str))
			    {
				// The user responded not to continue
				if (GUI.showRTM(msg) == 1) 
				    buffer[0] = 1;
				// The user wants to continue with the installation 
				else 
				    buffer[0] = 0;
			    }
			    // This violation is not to be displayed so send a 0 in the buffer[0] 
			    else
			    {
				System.out.println("Runtime violation suppressed by user selection : " + msg);
				buffer[0] = 0;
			    }
			}
			else 
			{
			    System.out.println("Runtime violation suppressed by script Analysis : " + msg);
			    buffer[0] = 0;
			}
			// create a datagram packet to send return message in
			dgPacket = new DatagramPacket(buffer,buffer.length, address, port);
			dgSocket.send(dgPacket);

		    }
		    else          
		    {
			System.out.println("Entered else part");
			proc.waitFor();
			try 
			{
			    int exitVal = proc.exitValue();
			    // The installation was successful
			    if (exitVal == 0) 
				upgradeResult = path + " : Installation was successful";
			    // The TRACER returned an error
			    else 
			    {
				System.out.println("The TRACER returned an abort message : " + exitVal);
				// This was an upgrade so we need to re-install the package overwritten
				if (installationType == 2)
				{
				    GUI.jlCurrentPackage.setText(path);
				    GUI.jlCurrentAction.setText("Attempting System Rollback..........");
				    GUI.pauseProgressBar();
				    String rollBackMsg = path + " : Installation was aborted."
					                 + "\nFor rollback,selection of the file installing the original package will now follow."; 
				    JOptionPane.showMessageDialog(null,rollBackMsg,"Attempting Rollback",JOptionPane.INFORMATION_MESSAGE); 
				    GUI.resumeProgressBar();

				    GUI.jlCurrentAction.setText("System Rollback in progress..........");
				    upgradeResult = rollBack(path);
				}
				else
				    upgradeResult = path + " : Installation was aborted.";
			    }
			    flag = 1;
			}
			catch (IllegalThreadStateException te) 
			{
			    System.out.println(te.getMessage());
			}
		    }// end of the else 
		}// end  of outer while(flag ==0)
		dgSocket.close();
	    } // end of rpm   execution try
	    
	    // rpm execution  try's catch block
	    catch (Exception e) 
	    {
		System.out.println(e.toString());
	    } 	    
	    //cleanup(path);
	} // end of outermost try

	catch(Exception e) 
	{
	    System.out.println(e.getMessage());
	}
	System.out.println(upgradeResult);
	return upgradeResult;
    }

    // This is where the rollback mechanism starts 
    private String rollBack(String path)
    {
	String msg = "";
	try
	{
	    String packagename = (String)(CommonMethods.getPartialPackageName(path)).elementAt(0);
	    String execStr = "rpm -e " + packagename;
	    System.out.println(execStr);
	    Process proc = theSystem.exec(execStr);
	    proc.waitFor();
	    int val = proc.exitValue();	    
	    if (val == 0)
	    {
		System.out.println("Package erased successfully");
		msg = installOldPackage(path);
	    }
	    else
	    { 
                BufferedReader buf = new BufferedReader(new InputStreamReader(proc.getErrorStream()));
		String s = "";
		try
		{
		    s = buf.readLine(); 
		}
		catch(IOException e)
		{
		    System.out.println("IO Exception in readline : " + e.getMessage());
		}
		if (!s.equals("") && !s.endsWith("is not installed"))
		{
		    System.out.println("Some error while rollback :" + s);
		    msg = errorRollBack();
		}
		else
		{
		    System.out.println("The package is not installed so no erasure was done");
		    msg = installOldPackage(path);
		}
		buf.close();
	    }
	}   
	catch(InterruptedException e)
	{
	    System.out.println("Interrupted Exception in proc.waitFor in rollBack: "  + e.getMessage());
	}
	catch(Exception e)
	{
	    System.out.println("Exception in method rollBack: "  + e.getMessage()); 
	}	
	finally
	{
	    return msg;
	} 
    }

    // Method to revert to the old installation by letting the user select the package
    private String installOldPackage(String newpath)
    {
	JDialog jdFile = new JDialog();      
	JFileChooser jfcChooser = new JFileChooser();
	jfcChooser.setDialogTitle("Please select the package to be installed");
	jfcChooser.setFileFilter(new RPMFileFilter());
	jfcChooser.removeChoosableFileFilter(jfcChooser.getAcceptAllFileFilter());
	GUI.pauseProgressBar();
	int returnVal = jfcChooser.showOpenDialog(jdFile);	
	GUI.resumeProgressBar();		    
	if(returnVal == JFileChooser.APPROVE_OPTION) 
	{
	    String path = jfcChooser.getSelectedFile().getAbsolutePath();
	    String msg = "";
	    try 
	    {
		String execStr = "rpm -i " + path;
		Process proc = theSystem.exec(execStr); 
		proc.waitFor();
		int val = proc.exitValue();
		// Normal termination 
		if (val == 0)
		{
		    System.out.println("Installation of old package successful");
		    msg = copyModifiedFiles(newpath);		    		    
		}
		// There was an installation error so return error message
		else
		{ 
		    System.out.println("Failure in installing old package");
		    msg = errorRollBack();
		}
	    }
	    catch (InterruptedException e) 
	    {
		System.out.println("Interrupted exception in doUpgrade " + e.getMessage());
	    }	
	    catch (Exception e)
	    {
		System.out.println("Exception in doUpgrade " + e.getMessage());
	    }
	    finally
	    {
		return msg;
	    }
	}
	else
	{
	    System.out.println("No file will be installed");
	    return "No package selected for re-installation.\n" +
		   "Please carry out the previous installation from the command prompt later.";
	}
    }

    // Method to copy modified files back to their respective locations
    private String copyModifiedFiles(String path)
    {
	String tarball = "/tmp/bkp-" + CommonMethods.getPackageNameWithVersion(path) + ".tgz";
	String msg = "";
	try
	{
	    // If backup file does not exist
	    if (!(new File(tarball)).exists())
	    {
		System.out.println("There are no backed up files to be copied. System rollback successful");
		msg = "System rollback successful.";
	    }
	    else
	    {
		// This sleep is deliberately inserted to modify the last modified time of these backed-up files so that 
		// the last modified time is always later than the time stored for these files in the rpm database
		Thread.sleep(1000);

		// We are trying to extract from a .tgz file with absolute path names , so z and P switches are required 
		String execStr = "tar -zxvP -f " + tarball;
		System.out.println(execStr);
		Process proc = theSystem.exec(execStr);
		proc.waitFor();
		int val = proc.exitValue();	    
		if (val == 0)
		{
		    BufferedReader buf = new BufferedReader(new InputStreamReader(proc.getInputStream()));
		    String s;
		    while ((s = buf.readLine()) != null) 
		    {
			File extracted = new File(s);
			extracted.setLastModified(System.currentTimeMillis());
		    }
		    System.out.println("Modified files copied. System rollback successful");
		    msg = "System rollback successful.";
		}
		else
		{ 
		    System.out.println("Error in extraction . System rollback failure");
		    msg = errorRollBack();
		}
	    }   
	}
	catch(InterruptedException e)
	{
	    System.out.println("Interrupted Exception in proc.waitFor in copyModifiedFiles: "  + e.getMessage());
	}
	catch(Exception e)
	{
	    System.out.println("Exception in method copyModifiedFiles: "  + e.getMessage()); 
	}	
	finally
	{
	    return msg;
	} 	
    }

    // Error message for Rollback
    private String errorRollBack()
    {
	return "There was a problem in system rollback.\n" +
	    "Please carry out the previous installation from the command prompt.";
    }	

    private void cleanup(String path)
    {
	String prefix = CommonMethods.getPackageNameWithVersion(path);
	File createFile =  new File ("/tmp/" + prefix + "-create");
	createFile.delete();
	File readFile = new File ("/tmp/" + prefix + "-read");
	readFile.delete();
	File writeFile = new File ("/tmp/" + prefix + "-write");
	writeFile.delete();
	File bkpFile = new File ("/tmp/bkp-" + prefix + ".tgz");
	bkpFile.delete();
	File preInstallScript = new File (Constants.DEMODIR + "scripts/" + prefix + ".preinstall.sh");
	preInstallScript.delete();
	File postInstallScript = new File (Constants.DEMODIR + "scripts/" + prefix + ".postinstall.sh");
	postInstallScript.delete();
	File preInstallDot = new File (Constants.DEMODIR + "scripts/" + prefix + ".preinstall.dot");
	preInstallDot.delete();
	File postInstallDot = new File (Constants.DEMODIR + "scripts/" + prefix + ".postinstall.dot");
	postInstallDot.delete();
	File preInstallPS = new File (Constants.DEMODIR + "scripts/" + prefix + ".preinstall.ps");
	preInstallPS.delete();
	File postInstallPS = new File (Constants.DEMODIR + "scripts/" + prefix + ".postinstall.ps");	                 
	postInstallPS.delete();
    }

    public static void main(String args[])
    {
	Vector origInstallList = new Vector();
 	System.out.println("Original List : ");
	for (int i = 0 ; i < args.length;i++)
	{
	    origInstallList.add(args[i]);
	    System.out.println(args[i]);
	}
	Upgrade obj = new Upgrade(origInstallList);
	System.out.println("\n");
	obj.sortInstallList();
    }

} //end class
